const
    Database = require('better-sqlite3'),
    logger = require("../lib/logger"),
    crypto = require('crypto'),
    eventLogger = require('../lib/eventLogger'),
    moment = require('moment')
;

const { v4: uuidv4 } = require('uuid');

moment.locale('hu');

let dbPath = process.env.MY_PATH + '/database/meatalyzer.sqlite';
//let dbPath =  '/home/pi/Kolbasz/web-ui/database/meatalyzer.sqlite';
let db = new Database(dbPath);
module.exports = db;

//region Table Update Time
function updateTableTime(tableName) {
    let template_tableUpdateTime = db.prepare('UPDATE tableUpdateTime set updateTime=(?) WHERE tableName=(?)');
    let tableUpdateTime_result = template_tableUpdateTime.run(Date.now(), tableName);
    logger.debug("tableUpdateTime: " + tableUpdateTime_result.changes);
}

module.exports.getTableUpdateTimes = function () {

    let sql = 'SELECT * FROM tableUpdateTime';
    let rows = db.prepare(sql).all();
    let tableUpdateTimes = {};

    rows.forEach((row) => {
        tableUpdateTimes[row.tableName] = row.updateTime;
    });

    return tableUpdateTimes;

};
//endregion

// region Users
function getUserJSONFromRow(row) {
    return {
        "username": row.username,
        "role": row.role,
        "displayName": row.displayName,
        "email": row.email,
        "studentId": row.studentId,
        "lastLogin": row.lastLogin
    };
}

function getUserRAWJSONFromRow(row) {
    return {
        "username": row.username,
        "role": row.role,
        "displayName": row.displayName,
        "email": row.email,
        "studentId": row.studentId,
        "lastLogin": row.lastLogin,
        "hash": row.myhash,
        "salt": row.mysalt
    };
}

module.exports.getUsers = function () {
    let rows = db.prepare("SELECT * FROM users").all();
    let users = [];

    rows.forEach((row) => {
        users.push(getUserJSONFromRow(row));
    });

    return users;
};

function getUser(username) {
    let rows = db.prepare("SELECT * FROM users WHERE username=(?)").all(username);
    let user = {};
    rows.forEach((row) => {user = getUserJSONFromRow(row);});
    return user;
}

function getUserRAW(username) {
    let rows = db.prepare("SELECT * FROM users WHERE username=(?)").all(username);
    let user = {};
    rows.forEach((row) => {user = getUserRAWJSONFromRow(row);});
    return user;
}

module.exports.getUser = function (username) {return getUser(username);};

module.exports.insertUser = function (username, password, role, displayName, email, studentId) {

    if (password !== "" && password != undefined) {
        crypto.randomBytes(32, function (err, buf) {
            if (err) {logger.error("error happened during crypto functions - insert user: " + err);}

            let salt = buf.toString('hex');

            crypto.pbkdf2(password, salt, 25000, 512, 'sha256', function (err, hashRaw) {
                if (err) {logger.error("error happened during crypto functions - insert user: " + err);}
                let hash = new Buffer(hashRaw, 'binary').toString('hex');

                let userTest = getUser(username);

                if (username === userTest.username) {
                    let sql = 'UPDATE users set role=(?), mysalt=(?), myhash=(?), displayName=(?), email=(?), updatedAt=(?) WHERE username=(?)';
                    let stmt_update = db.prepare(sql);
                    stmt_update.run(role, salt, hash, displayName, email, new Date().toLocaleString("hu-HU"), username);
                    logger.debug("Updated");
                } else {
                    let stmt_insert = db.prepare('INSERT INTO users VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)');
                    stmt_insert.run(username, role, hash, salt, new Date().toLocaleString("hu-HU"), new Date().toLocaleString("hu-HU"), displayName, email, studentId, 0);
                    logger.debug("Inserted");
                }

                updateTableTime("user");
            });
        });
    } else {
        let sql = 'UPDATE users set role=(?), displayName=(?), email=(?), updatedAt=(?) WHERE username=(?)';
        let stmt_update = db.prepare(sql);
        stmt_update.run(role, displayName, email, new Date().toLocaleString("en-US"), username);
        logger.debug("Updated");
        updateTableTime("user");
    }
};

module.exports.deleteUser = function (username) {
    let stmt = db.prepare('DELETE FROM users WHERE username=(?)');
    stmt.run(username);
    updateTableTime("user");
};

module.exports.updateUserLoginDate = function (username) {
    let stmt_update = db.prepare('UPDATE users set lastLogin=(?) WHERE username=(?)');
    stmt_update.run(Date.now(), username);
    updateTableTime("user");
};

module.exports.getUsersByRole = function (role) {
    let rows = db.prepare('SELECT * FROM users WHERE role=(?)').all(role);
    let results = [];
    rows.forEach((row) => {
        results.push(getUserJSONFromRow(row));
    });
    return results;
};

module.exports.getUsersByStudentIds = function (studentIds) {
    if (Array.isArray(studentIds)) studentIds = JSON.stringify(studentIds).slice(1, -1);
    let rows = db.prepare('SELECT * FROM users WHERE studentId in (' + studentIds + ')').all();
    let results = [];
    rows.forEach((row) => {
        results.push(getUserJSONFromRow(row));
    });
    return results;
};

module.exports.updateUserPassword = function (username, password) {
    if (password !== "" && password != undefined) {
        crypto.randomBytes(32, function (err, buf) {
            if (err) {logger.error("error happened during crypto functions - insert user: " + err);}

            let salt = buf.toString('hex');

            crypto.pbkdf2(password, salt, 25000, 512, 'sha256', function (err, hashRaw) {
                if (err) {logger.error("error happened during crypto functions - insert user: " + err);}
                let hash = new Buffer(hashRaw, 'binary').toString('hex');

                let userTest = getUser(username);

                if (username === userTest.username) {
                    try {
                        let sql = 'UPDATE users set mysalt=(?), myhash=(?), updatedAt=(?) WHERE username=(?)';
                        let stmt_update = db.prepare(sql);
                        stmt_update.run(salt, hash, Date.now(),  username);
                        logger.debug("Updated");
                        updateTableTime("user");
                    }
                    catch (e) {
                        logger.error(e);
                    }
                }
            });
        });
    }
};


// The default option values
var passordOptions = {
    activationkeylen:  8,
    resetPasswordkeylen:  8,
    saltlen:  32,
    iterations:  12000,
    keylen:  512,
    digest:  'sha1',
    hashField: 'hash',
    saltField: 'salt',
};

module.exports.modifyUserPassword = function (username, oldPassword, newPassword) {

    let userTest = getUserRAW(username);

    if (username === userTest.username && oldPassword !== "" && oldPassword !== undefined && newPassword !== "" && newPassword !== undefined) {

        crypto.pbkdf2(oldPassword, passordOptions.saltField, passordOptions.iterations, passordOptions.keylen, passordOptions.digest, function (err, hashRaw) {
            if (err) {return err;}

            var hash = new Buffer(hashRaw, 'binary').toString('hex');

            if (hash === userTest.hash) {

                crypto.randomBytes(32, function (err, buf) {
                    if (err) {logger.error("error happened during crypto functions - insert user: " + err);}

                    let newSalt = buf.toString('hex');

                    crypto.pbkdf2(newPassword, newSalt, 25000, 512, 'sha256', function (err, newHashRaw) {
                        if (err) {logger.error("error happened during crypto functions - insert user: " + err);}
                        let newHash = new Buffer(newHashRaw, 'binary').toString('hex');

                        if (username === userTest.username) {
                            let stmt_update = db.prepare('UPDATE users set mysalt=(?), myhash=(?) updatedAt=(?) WHERE username=(?)');
                            stmt_update.run(newSalt, newHash, Date.now(), username);
                            logger.debug("Updated");
                        }

                        updateTableTime("user");
                    });
                });
            }
            else {return { message: "incorrect Password" } }
        });
    }
};
// endregion

// region KVPs
module.exports.getKVPs = function () {

    let sql = 'SELECT * FROM kvp';
    let rows = db.prepare(sql).all();
    let kvps = {};

    rows.forEach((row) => {
        kvps[row.key] = row.value;
    });

    return kvps;

};

module.exports.updateKVPs = function (kvps) {
    for (let key in kvps) {
        let result = db.prepare('UPDATE kvp set value=(?) WHERE key=(?)').run(kvps[key], key);

        if(result.changes === 0) {result = db.prepare('INSERT INTO kvp VALUES (?, ?)').run(key, kvps[key]);}
    }

    updateTableTime("kvp");
};
// endregion

// region Event
function getEventJSONFromRow(row) {
    return {
        "eventId": row.eventId,
        "eventDate": row.eventDate,
        "referenceId": row.referenceId,
        "userId": row.userId,
        "type": row.type,
        "operation": row.operation,
        "result": row.result,
        "message": row.message,
    };
}

module.exports.getEvents = function () {
    let rows = db.prepare('SELECT * FROM event').all();
    let results = [];
    rows.forEach((row) => {
        results.push(getEventJSONFromRow(row));
    });
    return results;
};

module.exports.getEventByReferenceId = function (referenceId) {
    let rows = db.prepare('SELECT * FROM event WHERE referenceId LIKE ("%" || ? || "%")').all(referenceId);
    let results = [];
    rows.forEach((row) => {
        results.push(getEventJSONFromRow(row));
    });
    return results;
};

module.exports.getEvent = function (eventId) {
    let sql = 'SELECT * FROM event WHERE eventId=(?)';
    let rows = db.prepare(sql).all(eventId);
    let result = {};
    rows.forEach((row) => {
        result = getEventJSONFromRow(row);
    });
    return result;
};

module.exports.insertEvent = function (userId, referenceId, type, operation, eventResult, message) {
    let eventId = uuidv4(),
        eventDate = Date.now(); //generate ID

    try {
        let template = db.prepare('INSERT INTO event VALUES (?, ?, ?, ?, ?, ?, ?, ?)');
        let result = template.run(eventId, eventDate, userId, referenceId, type.value, operation.value, eventResult.value, message);
        logger.debug("insert into event: " + result.changes);
        updateTableTime("event");
    }
    catch (e) {
        logger.error(e);
    }

};
// endregion Event



// region DecaWave
function getRawDecaWaveDataJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "msgType":      row.msgType,
        "srcType":      row.srcType,
        "anchor":       row.anchor,
        "tag":          row.tag,
        "range0":       row.range0,
        "range1":       row.range1,
        "range2":       row.range2,
        "range3":       row.range3,
        "pos":          row.pos,
        "rawData":      row.rawData,
    };
}

function getDecaWaveTagPositionJSONFromRow(row) {
    return {
        "timeStamp": row.ts,
        "msgType":   row.msgType,
        "tag":       row.tag,
        "pos":       row.pos,
    };
}

module.exports.getDecaWaveRawData = function () {
    let rows = db.prepare('SELECT * FROM decaWave').all();
    let results = [];
    rows.forEach((row) => {results.push(getRawDecaWaveDataJSONFromRow(row));});
    return results;
};

module.exports.getDecaWaveTagPositionsBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT ts, msgType, tag, pos FROM decaWave WHERE ts>=(?) AND ts<=(?) AND srcType=(?)').all(fromTS, toTS, "Tag");
    let results = [];
    rows.forEach((row) => {results.push(getDecaWaveTagPositionJSONFromRow(row));});
    return results;
};

module.exports.getDecaWaveTagPositionsAfterTimeStamp = function (timeStamp/*, msgType*/) {
    //let rows = db.prepare('SELECT ts, tag, pos FROM decaWave WHERE ts>=(?) AND srcType=(?) AND msgType=(?)').all(timeStamp, "Tag", msgType);
    let rows = db.prepare('SELECT ts, msgType, tag, pos FROM decaWave WHERE ts>=(?) AND srcType=(?)').all(timeStamp, "Tag");
    let results = [];
    rows.forEach((row) => {results.push(getDecaWaveTagPositionJSONFromRow(row));});
    return results;
};

module.exports.insertDecaWaveRawData = function (timeStamp, msgType, srcType, anchor, tag, range0, range1, range2, range3, position, rawData) {
    try {
        db.prepare('INSERT INTO decaWave VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)').run(timeStamp, msgType, srcType, anchor, tag, range0, range1, range2, range3, position, rawData);
        updateTableTime("decaWave");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion DecaWave

//region Atmospheric Sensor
// region Atmospheric Data
function getAtmosphericDataJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "temperature":  row.temperature,
        "humidity":     row.humidity,
        "pressure":     row.pressure,
    };
}

module.exports.getAtmosphericData = function () {
    let rows = db.prepare('SELECT * FROM atmosphericData').all();
    let results = [];
    rows.forEach((row) => {results.push(getAtmosphericDataJSONFromRow(row));});
    return results;
};

module.exports.getAtmosphericDataBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM atmosphericData WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getAtmosphericDataJSONFromRow(row));});
    return results;
};

module.exports.getAtmosphericDataAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM atmosphericData WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getAtmosphericDataJSONFromRow(row));});
    return results;
};

module.exports.getAtmosphericDataOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM atmosphericData WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getAtmosphericDataJSONFromRow(row));});
    return results;
};

module.exports.getAtmosphericDataOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM atmosphericData WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getAtmosphericDataJSONFromRow(row));});
    return results;
};

module.exports.getAtmosphericDataOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM atmosphericData WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getAtmosphericDataJSONFromRow(row));});
    return results;
};

module.exports.insertAtmosphericData = function (timeStamp, deviceId, temperature, humidity, pressure) {
    try {
        db.prepare('INSERT INTO atmosphericData VALUES (?, ?, ?, ?, ?)').run(timeStamp, deviceId, temperature, humidity, pressure);
        updateTableTime("atmosphericData");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Atmospheric Data

// region Temperature
function getTemperatureJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "temperature":  row.temperature
    };
}

module.exports.getTemperature = function () {
    let rows = db.prepare('SELECT * FROM temperature').all();
    let results = [];
    rows.forEach((row) => {results.push(getTemperatureJSONFromRow(row));});
    return results;
};

module.exports.getTemperatureBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM temperature WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getTemperatureJSONFromRow(row));});
    return results;
};

module.exports.getTemperatureAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM temperature WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getTemperatureJSONFromRow(row));});
    return results;
};

module.exports.getTemperatureOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM temperature WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getTemperatureJSONFromRow(row));});
    return results;
};

module.exports.getTemperatureOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getTemperatureJSONFromRow(row));});
    return results;
};

module.exports.getTemperatureOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getTemperatureJSONFromRow(row));});
    return results;
};

module.exports.insertTemperature = function (timeStamp, deviceId, temperature) {
    try {
        db.prepare('INSERT INTO temperature VALUES (?, ?, ?)').run(timeStamp, deviceId, temperature);
        updateTableTime("temperature");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Temperature

// region Humidity
function getHumidityJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "humidity":     row.humidity,
    };
}

module.exports.getHumidity = function () {
    let rows = db.prepare('SELECT * FROM humidity').all();
    let results = [];
    rows.forEach((row) => {results.push(getHumidityJSONFromRow(row));});
    return results;
};

module.exports.getHumidityBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM humidity WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getHumidityJSONFromRow(row));});
    return results;
};

module.exports.getHumidityAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM humidity WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getHumidityJSONFromRow(row));});
    return results;
};

module.exports.getHumidityOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM humidity WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHumidityJSONFromRow(row));});
    return results;
};

module.exports.getHumidityOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM humidity WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHumidityJSONFromRow(row));});
    return results;
};

module.exports.getHumidityOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM humidity WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHumidityJSONFromRow(row));});
    return results;
};

module.exports.insertHumidity = function (timeStamp, deviceId, humidity) {
    try {
        db.prepare('INSERT INTO humidity VALUES (?, ?, ?)').run(timeStamp, deviceId, humidity);
        updateTableTime("humidity");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Humidity

// region Pressure
function getPressureJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "pressure":     row.pressure,
    };
}

module.exports.getPressure = function () {
    let rows = db.prepare('SELECT * FROM pressure').all();
    let results = [];
    rows.forEach((row) => {results.push(getPressureJSONFromRow(row));});
    return results;
};

module.exports.getPressureBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM pressure WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getPressureJSONFromRow(row));});
    return results;
};

module.exports.getPressureAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM pressure WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getPressureJSONFromRow(row));});
    return results;
};

module.exports.getPressureOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM pressure WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPressureJSONFromRow(row));});
    return results;
};

module.exports.getPressureOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM pressure WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPressureJSONFromRow(row));});
    return results;
};

module.exports.getPressureOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM pressure WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPressureJSONFromRow(row));});
    return results;
};

module.exports.insertPressure = function (timeStamp, deviceId, pressure) {
    try {
        db.prepare('INSERT INTO pressure VALUES (?, ?, ?)').run(timeStamp, deviceId, pressure);
        updateTableTime("pressure");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Pressure
//endregion Atmospheric Sensor

//region Spectroscopy
// region Spect_A
function getSpect_AJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_A":     row.value,
    };
}

module.exports.getSpect_A = function () {
    let rows = db.prepare('SELECT * FROM spect_A').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_AJSONFromRow(row));});
    return results;
};

module.exports.getSpect_ABetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_A WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_AJSONFromRow(row));});
    return results;
};

module.exports.getSpect_AAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_A WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_AJSONFromRow(row));});
    return results;
};

module.exports.getSpect_AOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_A WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_AJSONFromRow(row));});
    return results;
};

module.exports.getSpect_AOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_A WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_AJSONFromRow(row));});
    return results;
};

module.exports.getSpect_AOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_A WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_AJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_A = function (timeStamp, deviceId, spect_A) {
    try {
        db.prepare('INSERT INTO spect_A VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_A);
        updateTableTime("spect_A");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_A

// region Spect_B
function getSpect_BJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_B":     row.value,
    };
}

module.exports.getSpect_B = function () {
    let rows = db.prepare('SELECT * FROM spect_B').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_BJSONFromRow(row));});
    return results;
};

module.exports.getSpect_BBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_B WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_BJSONFromRow(row));});
    return results;
};

module.exports.getSpect_BAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_B WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_BJSONFromRow(row));});
    return results;
};

module.exports.getSpect_BOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_B WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_BJSONFromRow(row));});
    return results;
};

module.exports.getSpect_BOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_B WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_BJSONFromRow(row));});
    return results;
};

module.exports.getSpect_BOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_B WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_BJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_B = function (timeStamp, deviceId, spect_B) {
    try {
        db.prepare('INSERT INTO spect_B VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_B);
        updateTableTime("spect_B");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_B

// region Spect_C
function getSpect_CJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_C":     row.value,
    };
}

module.exports.getSpect_C = function () {
    let rows = db.prepare('SELECT * FROM spect_C').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_CJSONFromRow(row));});
    return results;
};

module.exports.getSpect_CBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_C WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_CJSONFromRow(row));});
    return results;
};

module.exports.getSpect_CAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_C WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_CJSONFromRow(row));});
    return results;
};

module.exports.getSpect_COfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_C WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_CJSONFromRow(row));});
    return results;
};

module.exports.getSpect_COfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_C WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_CJSONFromRow(row));});
    return results;
};

module.exports.getSpect_COfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_C WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_CJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_C = function (timeStamp, deviceId, spect_C) {
    try {
        db.prepare('INSERT INTO spect_C VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_C);
        updateTableTime("spect_C");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_C

// region Spect_D
function getSpect_DJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_D":     row.value,
    };
}

module.exports.getSpect_D = function () {
    let rows = db.prepare('SELECT * FROM spect_D').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_DJSONFromRow(row));});
    return results;
};

module.exports.getSpect_DBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_D WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_DJSONFromRow(row));});
    return results;
};

module.exports.getSpect_DAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_D WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_DJSONFromRow(row));});
    return results;
};

module.exports.getSpect_DOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_D WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_DJSONFromRow(row));});
    return results;
};

module.exports.getSpect_DOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_D WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_DJSONFromRow(row));});
    return results;
};

module.exports.getSpect_DOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_D WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_DJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_D = function (timeStamp, deviceId, spect_D) {
    try {
        db.prepare('INSERT INTO spect_D VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_D);
        updateTableTime("spect_D");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_D

// region Spect_E
function getSpect_EJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_E":     row.value,
    };
}

module.exports.getSpect_E = function () {
    let rows = db.prepare('SELECT * FROM spect_E').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_EJSONFromRow(row));});
    return results;
};

module.exports.getSpect_EBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_E WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_EJSONFromRow(row));});
    return results;
};

module.exports.getSpect_EAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_E WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_EJSONFromRow(row));});
    return results;
};

module.exports.getSpect_EOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_E WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_EJSONFromRow(row));});
    return results;
};

module.exports.getSpect_EOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_E WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_EJSONFromRow(row));});
    return results;
};

module.exports.getSpect_EOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_E WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_EJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_E = function (timeStamp, deviceId, spect_E) {
    try {
        db.prepare('INSERT INTO spect_E VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_E);
        updateTableTime("spect_E");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_E

// region Spect_F
function getSpect_FJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_F":     row.value,
    };
}

module.exports.getSpect_F = function () {
    let rows = db.prepare('SELECT * FROM spect_F').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_FJSONFromRow(row));});
    return results;
};

module.exports.getSpect_FBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_F WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_FJSONFromRow(row));});
    return results;
};

module.exports.getSpect_FAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_F WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_FJSONFromRow(row));});
    return results;
};

module.exports.getSpect_FOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_F WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_FJSONFromRow(row));});
    return results;
};

module.exports.getSpect_FOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_F WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_FJSONFromRow(row));});
    return results;
};

module.exports.getSpect_FOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_F WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_FJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_F = function (timeStamp, deviceId, spect_F) {
    try {
        db.prepare('INSERT INTO spect_F VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_F);
        updateTableTime("spect_F");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_F


// region Spect_G
function getSpect_GJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_G":     row.value,
    };
}

module.exports.getSpect_G = function () {
    let rows = db.prepare('SELECT * FROM spect_G').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_GJSONFromRow(row));});
    return results;
};

module.exports.getSpect_GBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_G WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_GJSONFromRow(row));});
    return results;
};

module.exports.getSpect_GAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_G WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_GJSONFromRow(row));});
    return results;
};

module.exports.getSpect_GOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_G WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_GJSONFromRow(row));});
    return results;
};

module.exports.getSpect_GOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_G WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_GJSONFromRow(row));});
    return results;
};

module.exports.getSpect_GOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_G WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_GJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_G = function (timeStamp, deviceId, spect_G) {
    try {
        db.prepare('INSERT INTO spect_G VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_G);
        updateTableTime("spect_G");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_G

// region Spect_H
function getSpect_HJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_H":     row.value,
    };
}

module.exports.getSpect_H = function () {
    let rows = db.prepare('SELECT * FROM spect_H').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_HJSONFromRow(row));});
    return results;
};

module.exports.getSpect_HBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_H WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_HJSONFromRow(row));});
    return results;
};

module.exports.getSpect_HAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_H WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_HJSONFromRow(row));});
    return results;
};

module.exports.getSpect_HOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_H WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_HJSONFromRow(row));});
    return results;
};

module.exports.getSpect_HOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_H WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_HJSONFromRow(row));});
    return results;
};

module.exports.getSpect_HOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_H WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_HJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_H = function (timeStamp, deviceId, spect_H) {
    try {
        db.prepare('INSERT INTO spect_H VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_H);
        updateTableTime("spect_H");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_H

// region Spect_I
function getSpect_IJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_I":     row.value,
    };
}

module.exports.getSpect_I = function () {
    let rows = db.prepare('SELECT * FROM spect_I').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_IJSONFromRow(row));});
    return results;
};

module.exports.getSpect_IBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_I WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_IJSONFromRow(row));});
    return results;
};

module.exports.getSpect_IAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_I WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_IJSONFromRow(row));});
    return results;
};

module.exports.getSpect_IOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_I WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_IJSONFromRow(row));});
    return results;
};

module.exports.getSpect_IOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_I WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_IJSONFromRow(row));});
    return results;
};

module.exports.getSpect_IOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_I WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_IJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_I = function (timeStamp, deviceId, spect_I) {
    try {
        db.prepare('INSERT INTO spect_I VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_I);
        updateTableTime("spect_I");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_I

// region Spect_J
function getSpect_JJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_J":     row.value,
    };
}

module.exports.getSpect_J = function () {
    let rows = db.prepare('SELECT * FROM spect_J').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_JJSONFromRow(row));});
    return results;
};

module.exports.getSpect_JBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_J WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_JJSONFromRow(row));});
    return results;
};

module.exports.getSpect_JAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_J WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_JJSONFromRow(row));});
    return results;
};

module.exports.getSpect_JOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_J WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_JJSONFromRow(row));});
    return results;
};

module.exports.getSpect_JOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_J WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_JJSONFromRow(row));});
    return results;
};

module.exports.getSpect_JOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_J WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_JJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_J = function (timeStamp, deviceId, spect_J) {
    try {
        db.prepare('INSERT INTO spect_J VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_J);
        updateTableTime("spect_J");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_J

// region Spect_K
function getSpect_KJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_K":     row.value,
    };
}

module.exports.getSpect_K = function () {
    let rows = db.prepare('SELECT * FROM spect_K').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_KJSONFromRow(row));});
    return results;
};

module.exports.getSpect_KBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_K WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_KJSONFromRow(row));});
    return results;
};

module.exports.getSpect_KAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_K WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_KJSONFromRow(row));});
    return results;
};

module.exports.getSpect_KOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_K WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_KJSONFromRow(row));});
    return results;
};

module.exports.getSpect_KOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_K WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_KJSONFromRow(row));});
    return results;
};

module.exports.getSpect_KOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_K WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_KJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_K = function (timeStamp, deviceId, spect_K) {
    try {
        db.prepare('INSERT INTO spect_K VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_K);
        updateTableTime("spect_K");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_K

// region Spect_L
function getSpect_LJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_L":     row.value,
    };
}

module.exports.getSpect_L = function () {
    let rows = db.prepare('SELECT * FROM spect_L').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_LJSONFromRow(row));});
    return results;
};

module.exports.getSpect_LBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_L WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_LJSONFromRow(row));});
    return results;
};

module.exports.getSpect_LAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_L WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_LJSONFromRow(row));});
    return results;
};

module.exports.getSpect_LOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_L WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_LJSONFromRow(row));});
    return results;
};

module.exports.getSpect_LOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_L WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_LJSONFromRow(row));});
    return results;
};

module.exports.getSpect_LOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_L WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_LJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_L = function (timeStamp, deviceId, spect_L) {
    try {
        db.prepare('INSERT INTO spect_L VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_L);
        updateTableTime("spect_L");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_L


// region Spect_R
function getSpect_RJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_R":     row.value,
    };
}

module.exports.getSpect_R = function () {
    let rows = db.prepare('SELECT * FROM spect_R').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_RJSONFromRow(row));});
    return results;
};

module.exports.getSpect_RBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_R WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_RJSONFromRow(row));});
    return results;
};

module.exports.getSpect_RAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_R WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_RJSONFromRow(row));});
    return results;
};

module.exports.getSpect_ROfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_R WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_RJSONFromRow(row));});
    return results;
};

module.exports.getSpect_ROfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_R WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_RJSONFromRow(row));});
    return results;
};

module.exports.getSpect_ROfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_R WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_RJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_R = function (timeStamp, deviceId, spect_R) {
    try {
        db.prepare('INSERT INTO spect_R VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_R);
        updateTableTime("spect_R");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_R

// region Spect_S
function getSpect_SJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_S":     row.value,
    };
}

module.exports.getSpect_S = function () {
    let rows = db.prepare('SELECT * FROM spect_S').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_SJSONFromRow(row));});
    return results;
};

module.exports.getSpect_SBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_S WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_SJSONFromRow(row));});
    return results;
};

module.exports.getSpect_SAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_S WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_SJSONFromRow(row));});
    return results;
};

module.exports.getSpect_SOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_S WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_SJSONFromRow(row));});
    return results;
};

module.exports.getSpect_SOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_S WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_SJSONFromRow(row));});
    return results;
};

module.exports.getSpect_SOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_S WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_SJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_S = function (timeStamp, deviceId, spect_S) {
    try {
        db.prepare('INSERT INTO spect_S VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_S);
        updateTableTime("spect_S");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_S

// region Spect_T
function getSpect_TJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_T":     row.value,
    };
}

module.exports.getSpect_T = function () {
    let rows = db.prepare('SELECT * FROM spect_T').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_TJSONFromRow(row));});
    return results;
};

module.exports.getSpect_TBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_T WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_TJSONFromRow(row));});
    return results;
};

module.exports.getSpect_TAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_T WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_TJSONFromRow(row));});
    return results;
};

module.exports.getSpect_TOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_T WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_TJSONFromRow(row));});
    return results;
};

module.exports.getSpect_TOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_T WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_TJSONFromRow(row));});
    return results;
};

module.exports.getSpect_TOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_T WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_TJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_T = function (timeStamp, deviceId, spect_T) {
    try {
        db.prepare('INSERT INTO spect_T VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_T);
        updateTableTime("spect_T");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_T

// region Spect_U
function getSpect_UJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_U":     row.value,
    };
}

module.exports.getSpect_U = function () {
    let rows = db.prepare('SELECT * FROM spect_U').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_UJSONFromRow(row));});
    return results;
};

module.exports.getSpect_UBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_U WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_UJSONFromRow(row));});
    return results;
};

module.exports.getSpect_UAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_U WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_UJSONFromRow(row));});
    return results;
};

module.exports.getSpect_UOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_U WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_UJSONFromRow(row));});
    return results;
};

module.exports.getSpect_UOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_U WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_UJSONFromRow(row));});
    return results;
};

module.exports.getSpect_UOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_U WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_UJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_U = function (timeStamp, deviceId, spect_U) {
    try {
        db.prepare('INSERT INTO spect_U VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_U);
        updateTableTime("spect_U");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_U

// region Spect_V
function getSpect_VJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_V":     row.value,
    };
}

module.exports.getSpect_V = function () {
    let rows = db.prepare('SELECT * FROM spect_V').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_VJSONFromRow(row));});
    return results;
};

module.exports.getSpect_VBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_V WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_VJSONFromRow(row));});
    return results;
};

module.exports.getSpect_VAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_V WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_VJSONFromRow(row));});
    return results;
};

module.exports.getSpect_VOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_V WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_VJSONFromRow(row));});
    return results;
};

module.exports.getSpect_VOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_V WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_VJSONFromRow(row));});
    return results;
};

module.exports.getSpect_VOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_V WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_VJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_V = function (timeStamp, deviceId, spect_V) {
    try {
        db.prepare('INSERT INTO spect_V VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_V);
        updateTableTime("spect_V");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_V

// region Spect_W
function getSpect_WJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "spect_W":     row.value,
    };
}

module.exports.getSpect_W = function () {
    let rows = db.prepare('SELECT * FROM spect_W').all();
    let results = [];
    rows.forEach((row) => {results.push(getSpect_WJSONFromRow(row));});
    return results;
};

module.exports.getSpect_WBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM spect_W WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_WJSONFromRow(row));});
    return results;
};

module.exports.getSpect_WAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM spect_W WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_WJSONFromRow(row));});
    return results;
};

module.exports.getSpect_WOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM spect_W WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_WJSONFromRow(row));});
    return results;
};

module.exports.getSpect_WOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_W WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_WJSONFromRow(row));});
    return results;
};

module.exports.getSpect_WOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM spect_W WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getSpect_WJSONFromRow(row));});
    return results;
};

module.exports.insertSpect_W = function (timeStamp, deviceId, spect_W) {
    try {
        db.prepare('INSERT INTO spect_W VALUES (?, ?, ?)').run(timeStamp, deviceId, spect_W);
        updateTableTime("spect_W");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Spect_W

//endregion Spectroscopy

//region Buttons
// region Button_Red
function getButton_RedJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "button_Red":   row.value,
    };
}

module.exports.getButton_Red = function () {
    let rows = db.prepare('SELECT * FROM button_Red').all();
    let results = [];
    rows.forEach((row) => {results.push(getButton_RedJSONFromRow(row));});
    return results;
};

module.exports.getButton_RedBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM button_Red WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getButton_RedJSONFromRow(row));});
    return results;
};

module.exports.getButton_RedAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM button_Red WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getButton_RedJSONFromRow(row));});
    return results;
};

module.exports.getButton_RedOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM button_Red WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getButton_RedJSONFromRow(row));});
    return results;
};

module.exports.getButton_RedOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM button_Red WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getButton_RedJSONFromRow(row));});
    return results;
};

module.exports.getButton_RedOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM button_Red WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getButton_RedJSONFromRow(row));});
    return results;
};

module.exports.insertButton_Red = function (timeStamp, deviceId, button_Red) {
    try {
        db.prepare('INSERT INTO button_Red VALUES (?, ?, ?)').run(timeStamp, deviceId, button_Red);
        updateTableTime("button_Red");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Button_Red

// region Button_Green
function getButton_GreenJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "button_Green": row.value,
    };
}

module.exports.getButton_Green = function () {
    let rows = db.prepare('SELECT * FROM button_Green').all();
    let results = [];
    rows.forEach((row) => {results.push(getButton_GreenJSONFromRow(row));});
    return results;
};

module.exports.getButton_GreenBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM button_Green WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getButton_GreenJSONFromRow(row));});
    return results;
};

module.exports.getButton_GreenAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM button_Green WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getButton_GreenJSONFromRow(row));});
    return results;
};

module.exports.getButton_GreenOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM button_Green WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getButton_GreenJSONFromRow(row));});
    return results;
};

module.exports.getButton_GreenOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM button_Green WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getButton_GreenJSONFromRow(row));});
    return results;
};

module.exports.getButton_GreenOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM button_Green WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getButton_GreenJSONFromRow(row));});
    return results;
};

module.exports.insertButton_Green = function (timeStamp, deviceId, button_Green) {
    try {
        db.prepare('INSERT INTO button_Green VALUES (?, ?, ?)').run(timeStamp, deviceId, button_Green);
        updateTableTime("button_Green");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Button_Green
//endregion Buttons

//region PH
// region PH_Value
function getPH_ValueJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "ph_Value":   row.value,
    };
}

module.exports.getPH_Value = function () {
    let rows = db.prepare('SELECT * FROM ph_Value').all();
    let results = [];
    rows.forEach((row) => {results.push(getPH_ValueJSONFromRow(row));});
    return results;
};

module.exports.getPH_ValueBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM ph_Value WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getPH_ValueJSONFromRow(row));});
    return results;
};

module.exports.getPH_ValueAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM ph_Value WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getPH_ValueJSONFromRow(row));});
    return results;
};

module.exports.getPH_ValueOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM ph_Value WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPH_ValueJSONFromRow(row));});
    return results;
};

module.exports.getPH_ValueOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM ph_Value WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPH_ValueJSONFromRow(row));});
    return results;
};

module.exports.getPH_ValueOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM ph_Value WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPH_ValueJSONFromRow(row));});
    return results;
};

module.exports.insertPH_Value = function (timeStamp, deviceId, ph_Value) {
    try {
        db.prepare('INSERT INTO ph_Value VALUES (?, ?, ?)').run(timeStamp, deviceId, ph_Value);
        updateTableTime("ph_Value");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion PH_Value

// region PH_Voltage
function getPH_VoltageJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "ph_Voltage": row.value,
    };
}

module.exports.getPH_Voltage = function () {
    let rows = db.prepare('SELECT * FROM ph_Voltage').all();
    let results = [];
    rows.forEach((row) => {results.push(getPH_VoltageJSONFromRow(row));});
    return results;
};

module.exports.getPH_VoltageBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM ph_Voltage WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getPH_VoltageJSONFromRow(row));});
    return results;
};

module.exports.getPH_VoltageAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM ph_Voltage WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getPH_VoltageJSONFromRow(row));});
    return results;
};

module.exports.getPH_VoltageOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM ph_Voltage WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPH_VoltageJSONFromRow(row));});
    return results;
};

module.exports.getPH_VoltageOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM ph_Voltage WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPH_VoltageJSONFromRow(row));});
    return results;
};

module.exports.getPH_VoltageOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM ph_Voltage WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPH_VoltageJSONFromRow(row));});
    return results;
};

module.exports.insertPH_Voltage = function (timeStamp, deviceId, ph_Voltage) {
    try {
        db.prepare('INSERT INTO ph_Voltage VALUES (?, ?, ?)').run(timeStamp, deviceId, ph_Voltage);
        updateTableTime("ph_Voltage");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion PH_Voltage
//endregion PH

//region Heart Rate
// region HR_confidence
function getHR_confidenceJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "hr_confidence":   row.value,
    };
}

module.exports.getHR_confidence = function () {
    let rows = db.prepare('SELECT * FROM hr_confidence').all();
    let results = [];
    rows.forEach((row) => {results.push(getHR_confidenceJSONFromRow(row));});
    return results;
};

module.exports.getHR_confidenceBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM hr_confidence WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getHR_confidenceJSONFromRow(row));});
    return results;
};

module.exports.getHR_confidenceAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM hr_confidence WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getHR_confidenceJSONFromRow(row));});
    return results;
};

module.exports.getHR_confidenceOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM hr_confidence WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_confidenceJSONFromRow(row));});
    return results;
};

module.exports.getHR_confidenceOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_confidence WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_confidenceJSONFromRow(row));});
    return results;
};

module.exports.getHR_confidenceOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_confidence WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_confidenceJSONFromRow(row));});
    return results;
};

module.exports.insertHR_confidence = function (timeStamp, deviceId, hr_confidence) {
    try {
        db.prepare('INSERT INTO hr_confidence VALUES (?, ?, ?)').run(timeStamp, deviceId, hr_confidence);
        updateTableTime("hr_confidence");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion HR_confidence

// region HR_extStatus
function getHR_extStatusJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "hr_extStatus": row.value,
    };
}

module.exports.getHR_extStatus = function () {
    let rows = db.prepare('SELECT * FROM hr_extStatus').all();
    let results = [];
    rows.forEach((row) => {results.push(getHR_extStatusJSONFromRow(row));});
    return results;
};

module.exports.getHR_extStatusBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM hr_extStatus WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getHR_extStatusJSONFromRow(row));});
    return results;
};

module.exports.getHR_extStatusAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM hr_extStatus WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getHR_extStatusJSONFromRow(row));});
    return results;
};

module.exports.getHR_extStatusOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM hr_extStatus WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_extStatusJSONFromRow(row));});
    return results;
};

module.exports.getHR_extStatusOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_extStatus WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_extStatusJSONFromRow(row));});
    return results;
};

module.exports.getHR_extStatusOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_extStatus WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_extStatusJSONFromRow(row));});
    return results;
};

module.exports.insertHR_extStatus = function (timeStamp, deviceId, hr_extStatus) {
    try {
        db.prepare('INSERT INTO hr_extStatus VALUES (?, ?, ?)').run(timeStamp, deviceId, hr_extStatus);
        updateTableTime("hr_extStatus");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion HR_extStatus

// region HR_heartRate
function getHR_heartRateJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "hr_heartRate":   row.value,
    };
}

module.exports.getHR_heartRate = function () {
    let rows = db.prepare('SELECT * FROM hr_heartRate').all();
    let results = [];
    rows.forEach((row) => {results.push(getHR_heartRateJSONFromRow(row));});
    return results;
};

module.exports.getHR_heartRateBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM hr_heartRate WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getHR_heartRateJSONFromRow(row));});
    return results;
};

module.exports.getHR_heartRateAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM hr_heartRate WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getHR_heartRateJSONFromRow(row));});
    return results;
};

module.exports.getHR_heartRateOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM hr_heartRate WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_heartRateJSONFromRow(row));});
    return results;
};

module.exports.getHR_heartRateOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_heartRate WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_heartRateJSONFromRow(row));});
    return results;
};

module.exports.getHR_heartRateOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_heartRate WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_heartRateJSONFromRow(row));});
    return results;
};

module.exports.insertHR_heartRate = function (timeStamp, deviceId, hr_heartRate) {
    try {
        db.prepare('INSERT INTO hr_heartRate VALUES (?, ?, ?)').run(timeStamp, deviceId, hr_heartRate);
        updateTableTime("hr_heartRate");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion HR_heartRate

// region HR_irLed
function getHR_irLedJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "hr_irLed": row.value,
    };
}

module.exports.getHR_irLed = function () {
    let rows = db.prepare('SELECT * FROM hr_irLed').all();
    let results = [];
    rows.forEach((row) => {results.push(getHR_irLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_irLedBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM hr_irLed WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getHR_irLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_irLedAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM hr_irLed WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getHR_irLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_irLedOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM hr_irLed WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_irLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_irLedOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_irLed WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_irLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_irLedOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_irLed WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_irLedJSONFromRow(row));});
    return results;
};

module.exports.insertHR_irLed = function (timeStamp, deviceId, hr_irLed) {
    try {
        db.prepare('INSERT INTO hr_irLed VALUES (?, ?, ?)').run(timeStamp, deviceId, hr_irLed);
        updateTableTime("hr_irLed");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion HR_irLed

// region HR_oxygen
function getHR_oxygenJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "hr_oxygen":   row.value,
    };
}

module.exports.getHR_oxygen = function () {
    let rows = db.prepare('SELECT * FROM hr_oxygen').all();
    let results = [];
    rows.forEach((row) => {results.push(getHR_oxygenJSONFromRow(row));});
    return results;
};

module.exports.getHR_oxygenBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM hr_oxygen WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getHR_oxygenJSONFromRow(row));});
    return results;
};

module.exports.getHR_oxygenAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM hr_oxygen WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getHR_oxygenJSONFromRow(row));});
    return results;
};

module.exports.getHR_oxygenOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM hr_oxygen WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_oxygenJSONFromRow(row));});
    return results;
};

module.exports.getHR_oxygenOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_oxygen WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_oxygenJSONFromRow(row));});
    return results;
};

module.exports.getHR_oxygenOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_oxygen WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_oxygenJSONFromRow(row));});
    return results;
};

module.exports.insertHR_oxygen = function (timeStamp, deviceId, hr_oxygen) {
    try {
        db.prepare('INSERT INTO hr_oxygen VALUES (?, ?, ?)').run(timeStamp, deviceId, hr_oxygen);
        updateTableTime("hr_oxygen");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion HR_oxygen

// region HR_redLed
function getHR_redLedJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "hr_redLed": row.value,
    };
}

module.exports.getHR_redLed = function () {
    let rows = db.prepare('SELECT * FROM hr_redLed').all();
    let results = [];
    rows.forEach((row) => {results.push(getHR_redLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_redLedBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM hr_redLed WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getHR_redLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_redLedAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM hr_redLed WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getHR_redLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_redLedOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM hr_redLed WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_redLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_redLedOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_redLed WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_redLedJSONFromRow(row));});
    return results;
};

module.exports.getHR_redLedOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_redLed WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_redLedJSONFromRow(row));});
    return results;
};

module.exports.insertHR_redLed = function (timeStamp, deviceId, hr_redLed) {
    try {
        db.prepare('INSERT INTO hr_redLed VALUES (?, ?, ?)').run(timeStamp, deviceId, hr_redLed);
        updateTableTime("hr_redLed");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion HR_redLed

// region HR_rValue
function getHR_rValueJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "hr_rValue": row.value,
    };
}

module.exports.getHR_rValue = function () {
    let rows = db.prepare('SELECT * FROM hr_rValue').all();
    let results = [];
    rows.forEach((row) => {results.push(getHR_rValueJSONFromRow(row));});
    return results;
};

module.exports.getHR_rValueBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM hr_rValue WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getHR_rValueJSONFromRow(row));});
    return results;
};

module.exports.getHR_rValueAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM hr_rValue WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getHR_rValueJSONFromRow(row));});
    return results;
};

module.exports.getHR_rValueOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM hr_rValue WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_rValueJSONFromRow(row));});
    return results;
};

module.exports.getHR_rValueOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_rValue WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_rValueJSONFromRow(row));});
    return results;
};

module.exports.getHR_rValueOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_rValue WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_rValueJSONFromRow(row));});
    return results;
};

module.exports.insertHR_rValue = function (timeStamp, deviceId, hr_rValue) {
    try {
        db.prepare('INSERT INTO hr_rValue VALUES (?, ?, ?)').run(timeStamp, deviceId, hr_rValue);
        updateTableTime("hr_rValue");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion HR_rValue

// region HR_status
function getHR_statusJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "hr_status": row.value,
    };
}

module.exports.getHR_status = function () {
    let rows = db.prepare('SELECT * FROM hr_status').all();
    let results = [];
    rows.forEach((row) => {results.push(getHR_statusJSONFromRow(row));});
    return results;
};

module.exports.getHR_statusBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM hr_status WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getHR_statusJSONFromRow(row));});
    return results;
};

module.exports.getHR_statusAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM hr_status WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getHR_statusJSONFromRow(row));});
    return results;
};

module.exports.getHR_statusOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM hr_status WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_statusJSONFromRow(row));});
    return results;
};

module.exports.getHR_statusOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_status WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_statusJSONFromRow(row));});
    return results;
};

module.exports.getHR_statusOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM hr_status WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getHR_statusJSONFromRow(row));});
    return results;
};

module.exports.insertHR_status = function (timeStamp, deviceId, hr_status) {
    try {
        db.prepare('INSERT INTO hr_status VALUES (?, ?, ?)').run(timeStamp, deviceId, hr_status);
        updateTableTime("hr_status");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion HR_status
//endregion Heart Rate

//region Scale
// region Scale_AvgWeight
function getScale_AvgWeightJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "scale_AvgWeight":   row.value,
    };
}

module.exports.getScale_AvgWeight = function () {
    let rows = db.prepare('SELECT * FROM scale_AvgWeight').all();
    let results = [];
    rows.forEach((row) => {results.push(getScale_AvgWeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_AvgWeightBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM scale_AvgWeight WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getScale_AvgWeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_AvgWeightAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM scale_AvgWeight WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getScale_AvgWeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_AvgWeightOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM scale_AvgWeight WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_AvgWeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_AvgWeightOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_AvgWeight WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_AvgWeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_AvgWeightOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_AvgWeight WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_AvgWeightJSONFromRow(row));});
    return results;
};

module.exports.insertScale_AvgWeight = function (timeStamp, deviceId, scale_AvgWeight) {
    try {
        db.prepare('INSERT INTO scale_AvgWeight VALUES (?, ?, ?)').run(timeStamp, deviceId, scale_AvgWeight);
        updateTableTime("scale_AvgWeight");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Scale_AvgWeight

// region Scale_CalibrationDate
function getScale_CalibrationDateJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "scale_CalibrationDate": row.value,
    };
}

module.exports.getScale_CalibrationDate = function () {
    let rows = db.prepare('SELECT * FROM scale_CalibrationDate').all();
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationDateJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationDateBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationDate WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationDateJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationDateAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationDate WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationDateJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationDateOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationDate WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationDateJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationDateOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationDate WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationDateJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationDateOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationDate WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationDateJSONFromRow(row));});
    return results;
};

module.exports.insertScale_CalibrationDate = function (timeStamp, deviceId, scale_CalibrationDate) {
    try {
        db.prepare('INSERT INTO scale_CalibrationDate VALUES (?, ?, ?)').run(timeStamp, deviceId, scale_CalibrationDate);
        updateTableTime("scale_CalibrationDate");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Scale_CalibrationDate

// region Scale_CalibrationFactor
function getScale_CalibrationFactorJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "scale_CalibrationFactor":   row.value,
    };
}

module.exports.getScale_CalibrationFactor = function () {
    let rows = db.prepare('SELECT * FROM scale_CalibrationFactor').all();
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationFactorJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationFactorBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationFactor WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationFactorJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationFactorAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationFactor WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationFactorJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationFactorOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationFactor WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationFactorJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationFactorOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationFactor WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationFactorJSONFromRow(row));});
    return results;
};

module.exports.getScale_CalibrationFactorOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_CalibrationFactor WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_CalibrationFactorJSONFromRow(row));});
    return results;
};

module.exports.insertScale_CalibrationFactor = function (timeStamp, deviceId, scale_CalibrationFactor) {
    try {
        db.prepare('INSERT INTO scale_CalibrationFactor VALUES (?, ?, ?)').run(timeStamp, deviceId, scale_CalibrationFactor);
        updateTableTime("scale_CalibrationFactor");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Scale_CalibrationFactor

// region Scale_Reading
function getScale_ReadingJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "scale_Reading": row.value,
    };
}

module.exports.getScale_Reading = function () {
    let rows = db.prepare('SELECT * FROM scale_Reading').all();
    let results = [];
    rows.forEach((row) => {results.push(getScale_ReadingJSONFromRow(row));});
    return results;
};

module.exports.getScale_ReadingBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM scale_Reading WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ReadingJSONFromRow(row));});
    return results;
};

module.exports.getScale_ReadingAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM scale_Reading WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ReadingJSONFromRow(row));});
    return results;
};

module.exports.getScale_ReadingOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM scale_Reading WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ReadingJSONFromRow(row));});
    return results;
};

module.exports.getScale_ReadingOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_Reading WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ReadingJSONFromRow(row));});
    return results;
};

module.exports.getScale_ReadingOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_Reading WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ReadingJSONFromRow(row));});
    return results;
};

module.exports.insertScale_Reading = function (timeStamp, deviceId, scale_Reading) {
    try {
        db.prepare('INSERT INTO scale_Reading VALUES (?, ?, ?)').run(timeStamp, deviceId, scale_Reading);
        updateTableTime("scale_Reading");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Scale_Reading

// region Scale_Weight
function getScale_WeightJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "scale_Weight":   row.value,
    };
}

module.exports.getScale_Weight = function () {
    let rows = db.prepare('SELECT * FROM scale_Weight').all();
    let results = [];
    rows.forEach((row) => {results.push(getScale_WeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_WeightBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM scale_Weight WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getScale_WeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_WeightAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM scale_Weight WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getScale_WeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_WeightOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM scale_Weight WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_WeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_WeightOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_Weight WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_WeightJSONFromRow(row));});
    return results;
};

module.exports.getScale_WeightOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_Weight WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_WeightJSONFromRow(row));});
    return results;
};

module.exports.insertScale_Weight = function (timeStamp, deviceId, scale_Weight) {
    try {
        db.prepare('INSERT INTO scale_Weight VALUES (?, ?, ?)').run(timeStamp, deviceId, scale_Weight);
        updateTableTime("scale_Weight");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Scale_Weight

// region Scale_ZeroOffset
function getScale_ZeroOffsetJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "scale_ZeroOffset": row.value,
    };
}

module.exports.getScale_ZeroOffset = function () {
    let rows = db.prepare('SELECT * FROM scale_ZeroOffset').all();
    let results = [];
    rows.forEach((row) => {results.push(getScale_ZeroOffsetJSONFromRow(row));});
    return results;
};

module.exports.getScale_ZeroOffsetBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM scale_ZeroOffset WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ZeroOffsetJSONFromRow(row));});
    return results;
};

module.exports.getScale_ZeroOffsetAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM scale_ZeroOffset WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ZeroOffsetJSONFromRow(row));});
    return results;
};

module.exports.getScale_ZeroOffsetOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM scale_ZeroOffset WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ZeroOffsetJSONFromRow(row));});
    return results;
};

module.exports.getScale_ZeroOffsetOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_ZeroOffset WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ZeroOffsetJSONFromRow(row));});
    return results;
};

module.exports.getScale_ZeroOffsetOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM scale_ZeroOffset WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getScale_ZeroOffsetJSONFromRow(row));});
    return results;
};

module.exports.insertScale_ZeroOffset = function (timeStamp, deviceId, scale_ZeroOffset) {
    try {
        db.prepare('INSERT INTO scale_ZeroOffset VALUES (?, ?, ?)').run(timeStamp, deviceId, scale_ZeroOffset);
        updateTableTime("scale_ZeroOffset");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion Scale_ZeroOffset
//endregion Scale

// region PWM
function getPWMJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "pwm":          row.value,
    };
}

module.exports.getPWM = function () {
    let rows = db.prepare('SELECT * FROM pwm').all();
    let results = [];
    rows.forEach((row) => {results.push(getPWMJSONFromRow(row));});
    return results;
};

module.exports.getPWMBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM pwm WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getPWMJSONFromRow(row));});
    return results;
};

module.exports.getPWMAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM pwm WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getPWMJSONFromRow(row));});
    return results;
};

module.exports.getPWMOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM pwm WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPWMJSONFromRow(row));});
    return results;
};

module.exports.getPWMOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM pwm WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPWMJSONFromRow(row));});
    return results;
};

module.exports.getPWMOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM pwm WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getPWMJSONFromRow(row));});
    return results;
};

module.exports.insertPWM = function (timeStamp, deviceId, pwm) {
    try {
        db.prepare('INSERT INTO pwm VALUES (?, ?, ?)').run(timeStamp, deviceId, pwm);
        updateTableTime("pwm");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion PWM

// region VoiceRecognition
function getVoiceRecognitionJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "voiceRecognition": row.value,
    };
}

module.exports.getVoiceRecognition = function () {
    let rows = db.prepare('SELECT * FROM voiceRecognition').all();
    let results = [];
    rows.forEach((row) => {results.push(getVoiceRecognitionJSONFromRow(row));});
    return results;
};

module.exports.getVoiceRecognitionBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM voiceRecognition WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getVoiceRecognitionJSONFromRow(row));});
    return results;
};

module.exports.getVoiceRecognitionAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM voiceRecognition WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getVoiceRecognitionJSONFromRow(row));});
    return results;
};

module.exports.getVoiceRecognitionOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM voiceRecognition WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getVoiceRecognitionJSONFromRow(row));});
    return results;
};

module.exports.getVoiceRecognitionOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM voiceRecognition WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getVoiceRecognitionJSONFromRow(row));});
    return results;
};

module.exports.getVoiceRecognitionOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM voiceRecognition WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getVoiceRecognitionJSONFromRow(row));});
    return results;
};

module.exports.insertVoiceRecognition = function (timeStamp, deviceId, voiceRecognition) {
    try {
        db.prepare('INSERT INTO voiceRecognition VALUES (?, ?, ?)').run(timeStamp, deviceId, voiceRecognition);
        updateTableTime("voiceRecognition");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion VoiceRecognition

//region Temperature
function gettemperature_AmbientJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "temperatureAmbient":   row.temperatureAmbient,
    };
}

function gettemperatureThermocoupleJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "temperatureThermocouple":   row.temperatureThermocouple,
    };
}

// region temperatureAmbient
module.exports.gettemperatureAmbient = function () {
    let rows = db.prepare('SELECT * FROM temperatureAmbient').all();
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureAmbientBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM temperatureAmbient WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureAmbientAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM temperatureAmbient WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureAmbientOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM temperatureAmbient WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureAmbientOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM temperatureAmbient WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureAmbientOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM temperatureAmbient WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.inserttemperatureAmbient = function (timeStamp, deviceId, temperatureAmbient) {
    try {
        db.prepare('INSERT INTO temperatureAmbient VALUES (?, ?, ?)').run(timeStamp, deviceId, temperatureAmbient);
        updateTableTime("temperatureAmbient");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion temperatureAmbient

// region temperature_B_Ambient
module.exports.gettemperature_B_Ambient = function () {
    let rows = db.prepare('SELECT * FROM temperature_B_Ambient').all();
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_AmbientBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM temperature_B_Ambient WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_AmbientAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM temperature_B_Ambient WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_AmbientOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_B_Ambient WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_AmbientOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_B_Ambient WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_AmbientOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_B_Ambient WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.inserttemperature_B_Ambient = function (timeStamp, deviceId, temperature_Ambient) {
    try {
        db.prepare('INSERT INTO temperature_B_Ambient VALUES (?, ?, ?)').run(timeStamp, deviceId, temperature_Ambient);
        updateTableTime("temperature_B_Ambient");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion temperature_B_Ambient

// region temperature_C_Ambient
module.exports.gettemperature_C_Ambient = function () {
    let rows = db.prepare('SELECT * FROM temperature_C_Ambient').all();
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_AmbientBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM temperature_C_Ambient WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_AmbientAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM temperature_C_Ambient WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_AmbientOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_C_Ambient WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_AmbientOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_C_Ambient WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_AmbientOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_C_Ambient WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperature_AmbientJSONFromRow(row));});
    return results;
};

module.exports.inserttemperature_C_Ambient = function (timeStamp, deviceId, temperature_Ambient) {
    try {
        db.prepare('INSERT INTO temperature_C_Ambient VALUES (?, ?, ?)').run(timeStamp, deviceId, temperature_Ambient);
        updateTableTime("temperature_C_Ambient");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion temperature_C_Ambient

// region temperatureThermocouple
module.exports.gettemperatureThermocouple = function () {
    let rows = db.prepare('SELECT * FROM temperatureThermocouple').all();
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureThermocoupleBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM temperatureThermocouple WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureThermocoupleAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM temperatureThermocouple WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureThermocoupleOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM temperatureThermocouple WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureThermocoupleOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM temperatureThermocouple WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperatureThermocoupleOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM temperatureThermocouple WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.inserttemperatureThermocouple = function (timeStamp, deviceId, temperatureThermocouple) {
    try {
        db.prepare('INSERT INTO temperatureThermocouple VALUES (?, ?, ?)').run(timeStamp, deviceId, temperatureThermocouple);
        updateTableTime("temperatureThermocouple");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion temperatureThermocouple

// region temperature_B_Thermocouple
module.exports.gettemperature_B_Thermocouple = function () {
    let rows = db.prepare('SELECT * FROM temperature_B_Thermocouple').all();
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_ThermocoupleBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM temperature_B_Thermocouple WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_ThermocoupleAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM temperature_B_Thermocouple WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_ThermocoupleOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_B_Thermocouple WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_ThermocoupleOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_B_Thermocouple WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_B_ThermocoupleOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_B_Thermocouple WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.inserttemperature_B_Thermocouple = function (timeStamp, deviceId, temperature_Thermocouple) {
    try {
        db.prepare('INSERT INTO temperature_B_Thermocouple VALUES (?, ?, ?)').run(timeStamp, deviceId, temperature_Thermocouple);
        updateTableTime("temperature_B_Thermocouple");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion temperature_B_Thermocouple

// region temperature_C_Thermocouple
module.exports.gettemperature_C_Thermocouple = function () {
    let rows = db.prepare('SELECT * FROM temperature_C_Thermocouple').all();
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_ThermocoupleBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM temperature_C_Thermocouple WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_ThermocoupleAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM temperature_C_Thermocouple WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_ThermocoupleOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_C_Thermocouple WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_ThermocoupleOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_C_Thermocouple WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.gettemperature_C_ThermocoupleOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM temperature_C_Thermocouple WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(gettemperatureThermocoupleJSONFromRow(row));});
    return results;
};

module.exports.inserttemperature_C_Thermocouple = function (timeStamp, deviceId, temperature_Thermocouple) {
    try {
        db.prepare('INSERT INTO temperature_C_Thermocouple VALUES (?, ?, ?)').run(timeStamp, deviceId, temperature_Thermocouple);
        updateTableTime("temperature_C_Thermocouple");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion temperature_C_Thermocouple
//endregion

//region Battery
// region voltage
function getvoltageJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "voltage":   row.voltage,
    };
}

module.exports.getvoltage = function () {
    let rows = db.prepare('SELECT * FROM voltage').all();
    let results = [];
    rows.forEach((row) => {results.push(getvoltageJSONFromRow(row));});
    return results;
};

module.exports.getvoltageBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM voltage WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getvoltageJSONFromRow(row));});
    return results;
};

module.exports.getvoltageAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM voltage WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getvoltageJSONFromRow(row));});
    return results;
};

module.exports.getvoltageOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM voltage WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getvoltageJSONFromRow(row));});
    return results;
};

module.exports.getvoltageOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM voltage WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getvoltageJSONFromRow(row));});
    return results;
};

module.exports.getvoltageOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM voltage WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getvoltageJSONFromRow(row));});
    return results;
};

module.exports.insertvoltage = function (timeStamp, deviceId, voltage) {
    try {
        db.prepare('INSERT INTO voltage VALUES (?, ?, ?)').run(timeStamp, deviceId, voltage);
        updateTableTime("voltage");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion voltage

// region current
function getcurrentJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "current":   row.current,
    };
}

module.exports.getcurrent = function () {
    let rows = db.prepare('SELECT * FROM current').all();
    let results = [];
    rows.forEach((row) => {results.push(getcurrentJSONFromRow(row));});
    return results;
};

module.exports.getcurrentBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM current WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getcurrentJSONFromRow(row));});
    return results;
};

module.exports.getcurrentAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM current WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getcurrentJSONFromRow(row));});
    return results;
};

module.exports.getcurrentOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM current WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getcurrentJSONFromRow(row));});
    return results;
};

module.exports.getcurrentOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM current WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getcurrentJSONFromRow(row));});
    return results;
};

module.exports.getcurrentOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM current WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getcurrentJSONFromRow(row));});
    return results;
};

module.exports.insertcurrent = function (timeStamp, deviceId, current) {
    try {
        db.prepare('INSERT INTO current VALUES (?, ?, ?)').run(timeStamp, deviceId, current);
        updateTableTime("current");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion current
//endregion

// region rfid
function getrfidJSONFromRow(row) {
    return {
        "timeStamp":    row.ts,
        "deviceId":     row.deviceId,
        "rfid": row.rfid,
    };
}

module.exports.getrfid = function () {
    let rows = db.prepare('SELECT * FROM rfid').all();
    let results = [];
    rows.forEach((row) => {results.push(getrfidJSONFromRow(row));});
    return results;
};

module.exports.getrfidBetweenTimeStamps = function (fromTS, toTS) {
    let rows = db.prepare('SELECT * FROM rfid WHERE ts>=(?) AND ts<=(?)').all(fromTS, toTS);
    let results = [];
    rows.forEach((row) => {results.push(getrfidJSONFromRow(row));});
    return results;
};

module.exports.getrfidAfterTimeStamp = function (timeStamp) {
    let rows = db.prepare('SELECT * FROM rfid WHERE ts>=(?)').all(timeStamp);
    let results = [];
    rows.forEach((row) => {results.push(getrfidJSONFromRow(row));});
    return results;
};

module.exports.getrfidOfDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM rfid WHERE deviceId=(?)').all(deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getrfidJSONFromRow(row));});
    return results;
};

module.exports.getrfidOfDeviceBetweenTimeStamps = function (fromTS, toTS, deviceId) {
    let rows = db.prepare('SELECT * FROM rfid WHERE ts>=(?) AND ts<=(?) AND deviceId=(?)').all(fromTS, toTS, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getrfidJSONFromRow(row));});
    return results;
};

module.exports.getrfidOfDeviceAfterTimeStamp = function (timeStamp, deviceId) {
    let rows = db.prepare('SELECT * FROM rfid WHERE ts>=(?) AND deviceId=(?)').all(timeStamp, deviceId);
    let results = [];
    rows.forEach((row) => {results.push(getrfidJSONFromRow(row));});
    return results;
};

module.exports.insertrfid = function (timeStamp, deviceId, rfid) {
    try {
        db.prepare('INSERT INTO rfid VALUES (?, ?, ?)').run(timeStamp, deviceId, rfid);
        updateTableTime("rfid");
    }
    catch (e) {
        logger.error(e);
    }
};

module.exports.deleteRFIDOfDevice = function (deviceId) {
    try {
        db.prepare('DELETE FROM rfid WHERE deviceId=(?)').run(deviceId);
        updateTableTime("rfid");
    }
    catch (e) {
        logger.error(e);
    }
};
// endregion rfid

//region Enum

// region Status
function getStatusJSONFromRow(row) {
    return {
        "id": row.id,
        "name": row.name,
        "description": row.description,
        "relevance": row.relevance,
    };
}
module.exports.getStatus = function (id) {
    let rows = db.prepare(`SELECT * FROM status WHERE id=(?)`).all(id);
    let status = {};
    rows.forEach((row) => {
        status = getStatusJSONFromRow(row);
    });
    return status;
};

module.exports.getStatuses = function () {
    let rows = db.prepare(`SELECT * FROM status`).all();
    let status = [];
    rows.forEach((row) => {
        status.push(getStatusJSONFromRow(row));
    });
    return status;
};

module.exports.getStatusesByRelevance = function (relevance) {
    let rows = db.prepare(`SELECT * FROM status WHERE relevance LIKE ("%" || ? || "%")`).all(relevance);
    let status = [];
    rows.forEach((row) => {status.push(getStatusJSONFromRow(row));});
    return status;
};

module.exports.insertStatus = function (id, name, description, relevance) {
    let result = db.prepare('INSERT INTO status VALUES (?, ?, ?, ?)').run(id, name, description, relevance);
    logger.debug("insert into status: " + result.changes);
    updateTableTime("status");
};

module.exports.updateStatus = function (id, name, description, relevance) {
    let result = db.prepare('UPDATE status SET name=(?), description=(?), relevance=(?) WHERE id=(?)').run(name, description, relevance, id);
    logger.debug("update status (name: " + name + "): " + result.changes);
    updateTableTime("status");
};

module.exports.deleteStatus = function (id) {
    db.prepare('DELETE FROM status WHERE id=(?)').run(id);
    logger.debug("delete status: " + id + ")");
    updateTableTime("status");
};
// endregion Status

// region Result
module.exports.getResult = function (id) {
    let rows = db.prepare(`SELECT * FROM result WHERE id=(?)`).all(id);
    let res = {};
    rows.forEach((row) => {
        res["id"] = row.id;
        res["name"] = row.name;
        res["description"] = row.description;
    });
    return res;
};

module.exports.getResults = function () {
    let rows = db.prepare(`SELECT * FROM result`).all();
    let res = [];
    rows.forEach((row) => {
        res.push({
            "id": row.id,
            "name": row.name,
            "description": row.description,
        });
    });
    return res;
};

module.exports.insertResult = function (id, name, description) {
    let res = db.prepare('INSERT INTO result VALUES (?, ?, ?)').run(id, name, description);
    logger.debug("insert into result: " + res.changes);
    updateTableTime("result");
};

module.exports.updateResult = function (id, name, description) {
    let res = db.prepare('UPDATE result SET name=(?), description=(?) WHERE id=(?)').run(name, description, id);
    logger.debug("update result (name: " + name + "): " + res.changes);
    updateTableTime("result");
};

module.exports.deleteResult = function (id) {
    db.prepare('DELETE FROM result WHERE id=(?)').run(id);
    logger.debug("delete result: " + id + ")");
    updateTableTime("result");
};
// endregion Result

// region Role
module.exports.getRole = function (id) {
    let rows = db.prepare(`SELECT * FROM role WHERE id=(?)`).all(id);
    let role = {};
    rows.forEach((row) => {
        role["id"] = row.id;
        role["name"] = row.name;
        role["description"] = row.description;
    });
    return role;
};

module.exports.getRoles = function () {
    let rows = db.prepare(`SELECT * FROM role`).all();
    let role = [];
    rows.forEach((row) => {
        role.push({
            "id": row.id,
            "name": row.name,
            "description": row.description,
        });
    });
    return role;
};

module.exports.insertRole = function (id, name, description) {
    let result = db.prepare('INSERT INTO role VALUES (?, ?, ?)').run(id, name, description);
    logger.debug("insert into role: " + result.changes);
    updateTableTime("role");
};

module.exports.updateRole = function (id, name, description) {
    let result = db.prepare('UPDATE role SET name=(?), description=(?) WHERE id=(?)').run(name, description, id);
    logger.debug("update role (name: " + name + "): " + result.changes);
    updateTableTime("role");
};

module.exports.deleteRole = function (id) {
    db.prepare('DELETE FROM role WHERE id=(?)').run(id);
    logger.debug("delete role: " + id + ")");
    updateTableTime("role");
};
// endregion Role

// region UserRole
module.exports.getUserRole = function (id) {
    let rows = db.prepare(`SELECT * FROM userRole WHERE id=(?)`).all(id);
    let role = {};
    rows.forEach((row) => {
        role["id"] = row.id;
        role["name"] = row.name;
        role["description"] = row.description;
    });
    return role;
};

module.exports.getUserRoles = function () {
    let rows = db.prepare(`SELECT * FROM userRole`).all();
    let role = [];
    rows.forEach((row) => {
        role.push({
            "id": row.id,
            "name": row.name,
            "description": row.description,
        });
    });
    return role;
};

module.exports.insertUserRole = function (id, name, description) {
    let result = db.prepare('INSERT INTO userRole VALUES (?, ?, ?)').run(id, name, description);
    logger.debug("insert into role: " + result.changes);
    updateTableTime("userRole");
};

module.exports.updateUserRole = function (id, name, description) {
    let result = db.prepare('UPDATE userRole SET name=(?), description=(?) WHERE id=(?)').run(name, description, id);
    logger.debug("update role (name: " + name + "): " + result.changes);
    updateTableTime("userRole");
};

module.exports.deleteUserRole = function (id) {
    db.prepare('DELETE FROM userRole WHERE id=(?)').run(id);
    logger.debug("delete role: " + id + ")");
    updateTableTime("userRole");
};
// endregion Role

// region Role
module.exports.getTag = function (id) {
    let rows = db.prepare(`SELECT * FROM tag WHERE id=(?)`).all(id);
    let role = {};
    rows.forEach((row) => {
        role["id"] = row.id;
        role["name"] = row.name;
        role["description"] = row.description;
    });
    return role;
};

module.exports.getTags = function () {
    let rows = db.prepare(`SELECT * FROM tag`).all();
    let role = [];
    rows.forEach((row) => {
        role.push({
            "id": row.id,
            "name": row.name,
            "description": row.description,
        });
    });
    return role;
};

module.exports.insertTag = function (id, name, description) {
    let result = db.prepare('INSERT INTO tag VALUES (?, ?, ?)').run(id, name, description);
    logger.debug("insert into tag: " + result.changes);
    updateTableTime("tag");
};

module.exports.updateTag = function (id, name, description) {
    let result = db.prepare('UPDATE tag SET name=(?), description=(?) WHERE id=(?)').run(name, description, id);
    logger.debug("update tag (name: " + name + "): " + result.changes);
    updateTableTime("tag");
};

module.exports.deleteTag = function (id) {
    db.prepare('DELETE FROM tag WHERE id=(?)').run(id);
    logger.debug("delete tag: " + id + ")");
    updateTableTime("tag");
};
// endregion Role

// region Event Operation
module.exports.getEventOperation = function (id) {
    let rows = db.prepare(`SELECT * FROM eventOperation WHERE id=(?)`).all(id);
    let category = {};
    rows.forEach((row) => {
        category["id"] = row.id;
        category["name"] = row.name;
        category["description"] = row.description;
    });
    return category;
};

module.exports.getEventOperations = function () {
    let rows = db.prepare(`SELECT * FROM eventOperation`).all();
    let category = [];
    rows.forEach((row) => {
        category.push({
            "id": row.id,
            "name": row.name,
            "description": row.description,
        });
    });
    return category;
};

module.exports.insertEventOperation = function (id, name, description) {
    let result = db.prepare('INSERT INTO eventOperation VALUES (?, ?, ?)').run(id, name, description);
    logger.debug("insert into eventOperation: " + result.changes);
    updateTableTime("eventOperation");
};

module.exports.updateEventOperation = function (id, name, description) {
    let result = db.prepare('UPDATE eventOperation SET name=(?), description=(?) WHERE id=(?)').run(name, description, id);
    logger.debug("update eventOperation (name: " + name + "): " + result.changes);
    updateTableTime("eventOperation");
};

module.exports.deleteEventOperation = function (id) {
    db.prepare('DELETE FROM eventOperation WHERE id=(?)').run(id);
    logger.debug("delete eventOperation: " + id + ")");
    updateTableTime("eventOperation");
};
// endregion Event Operation

// region Event Result
module.exports.getEventResult = function (id) {
    let rows = db.prepare(`SELECT * FROM eventResult WHERE id=(?)`).all(id);
    let category = {};
    rows.forEach((row) => {
        category["id"] = row.id;
        category["name"] = row.name;
        category["description"] = row.description;
    });
    return category;
};

module.exports.getEventResults = function () {
    let rows = db.prepare(`SELECT * FROM eventResult`).all();
    let category = [];
    rows.forEach((row) => {
        category.push({
            "id": row.id,
            "name": row.name,
            "description": row.description,
        });
    });
    return category;
};

module.exports.insertEventResult = function (id, name, description) {
    let result = db.prepare('INSERT INTO eventResult VALUES (?, ?, ?)').run(id, name, description);
    logger.debug("insert into eventResult: " + result.changes);
    updateTableTime("eventResult");
};

module.exports.updateEventResult = function (id, name, description) {
    let result = db.prepare('UPDATE eventResult SET name=(?), description=(?) WHERE id=(?)').run(name, description, id);
    logger.debug("update eventResult (name: " + name + "): " + result.changes);
    updateTableTime("eventResult");
};

module.exports.deleteEventResult = function (id) {
    db.prepare('DELETE FROM eventResult WHERE id=(?)').run(id);
    logger.debug("delete eventResult: " + id + ")");
    updateTableTime("eventResult");
};
// endregion Event Operation

// region DeviceType
module.exports.getDeviceType = function (id) {
    let rows = db.prepare(`SELECT * FROM deviceType WHERE id=(?)`).all(id);
    let deviceType = {};
    rows.forEach((row) => {
        deviceType["id"] = row.id;
        deviceType["name"] = row.name;
        deviceType["description"] = row.description;
    });
    return deviceType;
};

module.exports.getDeviceTypes = function () {
    let rows = db.prepare(`SELECT * FROM deviceType`).all();
    let deviceType = [];
    rows.forEach((row) => {
        deviceType.push({
            "id": row.id,
            "name": row.name,
            "description": row.description,
        });
    });
    return deviceType;
};

module.exports.insertDeviceType = function (id, name, description) {
    let result = db.prepare('INSERT INTO deviceType VALUES (?, ?, ?)').run(id, name, description);
    logger.debug("insert into deviceType: " + result.changes);
    updateTableTime("deviceType");
};

module.exports.updateDeviceType = function (id, name, description) {
    let result = db.prepare('UPDATE deviceType SET name=(?), description=(?) WHERE id=(?)').run(name, description, id);
    logger.debug("update deviceType (name: " + name + "): " + result.changes);
    updateTableTime("deviceType");
};

module.exports.deleteDeviceType = function (id) {
    db.prepare('DELETE FROM deviceType WHERE id=(?)').run(id);
    logger.debug("delete deviceType: " + id + ")");
    updateTableTime("deviceType");
};
// endregion Role

//endregion Enum


// region Device
function getDeviceJSONFromRow(row) {
    return {
        "deviceId": row.deviceId,
        "deviceName": row.deviceName,
        "connectionStatus": row.connectionStatus,
        "deviceType": row.deviceType,
        "MAC_Address": row.MAC_address,
    };
}

module.exports.getDevices = function () {
    let rows = db.prepare("SELECT * FROM device").all();
    let devices = [];
    rows.forEach((row) => {devices.push(getDeviceJSONFromRow(row));});
    return devices;
};

module.exports.getDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM device WHERE deviceId=(?)').all(deviceId);
    let device;
    rows.forEach((row) => {device = getDeviceJSONFromRow(row);});
    return device;
};

module.exports.getDevicesByDeviceType = function (deviceType) {
    let rows = db.prepare('SELECT * FROM device WHERE deviceType=(?)').all(deviceType);
    let devices = [];
    rows.forEach((row) => {devices.push(getDeviceJSONFromRow(row));});
    return devices;
};

module.exports.insertDevice = function (deviceName, deviceType, MAC_address) {
    let deviceId = uuidv4();
    db.prepare('INSERT INTO device VALUES (?, ?, ?, ?, ?)').run(deviceId, deviceName, "UNKNOWN", deviceType, MAC_address);
    updateTableTime("device");
};

module.exports.deleteDevice = function (deviceId) {
    db.prepare('DELETE FROM device WHERE deviceId=(?)').run(deviceId);
    updateTableTime("device");
};
module.exports.deleteWrongDevices = function () {
    db.prepare('DELETE FROM device WHERE deviceName=(?)').run("Szekér_1");
    updateTableTime("device");
};

module.exports.modifyDevice = function (deviceId, deviceName, deviceType, MAC_address) {
    try {
        db.prepare('UPDATE device set deviceName=(?), deviceType=(?), MAC_address=(?) WHERE deviceId=(?)').run( deviceName, deviceType, MAC_address, deviceId);
        updateTableTime("device");
    } catch (e) {
        console.log(e);
    }
};

module.exports.updateDeviceId = function (deviceId_old, deviceId_new) {
    try {
        db.prepare('UPDATE device set deviceId=(?) WHERE deviceId=(?)').run( deviceId_new, deviceId_old);
        updateTableTime("device");
    } catch (e) {
        console.log(e);
    }

};

module.exports.updateDeviceIdBasedOnMacAddress = function (MACAddress, deviceId_new) {
    try {
        db.prepare('UPDATE device set deviceId=(?) WHERE MAC_address=(?)').run( deviceId_new, MACAddress);
        updateTableTime("device");
    } catch (e) {
        console.log(e);
    }

};
// endregion

// region RFID_Device
function getRFIDDeviceJSONFromRow(row) {
    return {
        "id": row.id,
        "deviceId": row.deviceId,
        "rfid": row.rfid,
        "timeStampFrom": row.tsFrom,
        "timeStampTo": row.tsTo,
    };
}

module.exports.getRFIDDevices = function () {
    let rows = db.prepare("SELECT * FROM rfid_device").all();
    let items = [];
    rows.forEach((row) => {items.push(getRFIDDeviceJSONFromRow(row));});
    return items;
};

module.exports.getRFIDDevice = function (deviceId) {
    let rows = db.prepare('SELECT * FROM rfid_device WHERE deviceId=(?)').all(deviceId);
    let items = [];
    rows.forEach((row) => {items.push(getRFIDDeviceJSONFromRow(row));});
    return items;
};

module.exports.getRFIDDeviceByTimeStamp = function (deviceId, timeStamp) {
    //let rows = db.prepare('SELECT * FROM rfid_device WHERE deviceId=(?) AND (?)>=tsFrom AND (?)<=tsTo').all(deviceId, timeStamp, timeStamp);
    let rows = db.prepare('SELECT * FROM rfid_device WHERE deviceId=(?)').all(deviceId);
    let items = [];
    rows.forEach((row) => {
        let temp = getRFIDDeviceJSONFromRow(row);
        if(timeStamp >= temp.timeStampFrom  && timeStamp <= temp.timeStampTo) items.push(temp);
    });
    return items;
};

module.exports.insertRFIDDevice = function (deviceId, rfid, timeStampFrom,timeStampTo) {
    let id = uuidv4();
    db.prepare('INSERT INTO rfid_device VALUES (?, ?, ?,?,?)').run(id, deviceId, rfid,timeStampFrom, timeStampTo);
    updateTableTime("rfid_device");
};

module.exports.deleteRFIDDevice = function (id) {
    db.prepare('DELETE FROM rfid_device WHERE id=(?)').run(id);
    updateTableTime("rfid_device");
};

module.exports.modifyRFIDDevice = function (id, deviceId, rfid, timeStampFrom, timeStampTo) {
    db.prepare('UPDATE rfid_device set deviceId=(?), rfid=(?), tsFrom=(?), tsTo=(?) WHERE id=(?)').run(deviceId, rfid, timeStampFrom, timeStampTo, id);
    updateTableTime("rfid_device");
};
// endregion