var express         = require('express'),
    middleware      = require('../middleware/index'),
    db              = require('../lib/SQLConnection'),
    eventLogger     = require('../lib/eventLogger'),
    dataManager     = require('../lib/dataManager'),
    moment          = require('moment'),
    cmd             = require('node-cmd'),
    fs              = require('fs')
;

var router = express.Router();

function tryParseInt(str, defaultValue) {
    let retValue = defaultValue;
    if(str !== null) {
        if(str.length > 0) {
            if (!isNaN(str)) {
                retValue = parseInt(str);
            }
        }
    }
    return retValue;
}

//region Users
router.get('/users', middleware.isLoggedIn, function (req, res) {
    res.status(200).json(db.getUsers());
});

router.get('/usersByRole', middleware.isLoggedIn, function (req, res) {
    let role = req.query.role;
    res.status(200).json(db.getUsersByRole(role));
});

router.get('/usersByStudentIds', middleware.isLoggedIn, function (req, res) {
    let studentIds = req.query.studentIds;
    if(studentIds != undefined) studentIds = JSON.parse(studentIds);
    res.status(200).json(db.getUsersByStudentIds(studentIds));
});

router.get('/user', middleware.isLoggedIn, function (req, res) {
    let username = req.query.username;

    if (username !== "") {
        let result = db.getUser(username);

        if (result.username === username) res.status(200).json(result);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});

    } else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.put('/user', middleware.isLoggedIn, function (req, res) {

    let username = req.query.username,
        password = req.query.password,
        displayName = req.query.displayName,
        email = req.query.email,
        role = req.query.role;

    if (username !== "" && role !== "") {
        db.insertUser(username, password, role, displayName, email);
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.put('/modifyUser', middleware.isLoggedIn, function (req, res) {
    let username = req.query.username,
        displayName = req.query.displayName,
        email = req.query.email,
        role = req.query.role;

    if (username !== "" && role !== "") {
        db.modifyUser(username, role, displayName, email);
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.put('/updateUserPassword', middleware.isLoggedIn, function (req, res) {
    let username = req.query.username,
        password = req.query.password;

    if (username !== "" && password !== "") {
        db.updateUserPassword(username, password);
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.delete('/user', middleware.isLoggedIn, function (req, res) {
    let username = req.query.username;

    if (username !== "") {
        db.deleteUser(username);
        res.status(200);
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});
//endregion

//region KVPs
router.get('/kvps', middleware.isLoggedIn, function (req, res) {
    res.status(200).json(db.getKVPs());
});

router.put('/kvps', middleware.isLoggedIn, function (req, res) {
    let kvps = req.body;

    if (kvps && kvps !== "") {
        db.updateKVPs(kvps);
        res.status(200).json({status: "Success"});
    }
    else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion

//region Event
router.get('/events', middleware.isLoggedIn, function (req, res) {
    let results = db.getEvents();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/eventsByReferenceId', middleware.isLoggedIn, function (req, res) {
    let referenceId = req.query.referenceId;

    if (referenceId !== "") {
        let results = db.getEventByReferenceId(referenceId);

        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});

    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/event', middleware.isLoggedIn, function (req, res) {
    let eventId = req.query.eventId;

    if (eventId !== "") {
        let result = db.getEvent(eventId);

        if (result.eventId === eventId) res.status(200).json(result);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});

    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});
//endregion Event

router.get('/everythingBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {

        let decaWaveTagPositionsBetweenTimeStamps = db.getDecaWaveTagPositionsBetweenTimeStamps(fromTS, toTS);
        let temperatureBetweenTimeStamps    = db.getTemperatureBetweenTimeStamps(fromTS, toTS);
        let humidityBetweenTimeStamps       = db.getHumidityBetweenTimeStamps(fromTS, toTS);
        let Spect_ABetweenTimeStamps        = db.getSpect_ABetweenTimeStamps(fromTS, toTS);
        let Spect_BBetweenTimeStamps        = db.getSpect_BBetweenTimeStamps(fromTS, toTS);
        let Spect_CBetweenTimeStamps        = db.getSpect_CBetweenTimeStamps(fromTS, toTS);
        let Spect_DBetweenTimeStamps        = db.getSpect_DBetweenTimeStamps(fromTS, toTS);
        let Spect_EBetweenTimeStamps        = db.getSpect_EBetweenTimeStamps(fromTS, toTS);
        let Spect_FBetweenTimeStamps        = db.getSpect_FBetweenTimeStamps(fromTS, toTS);
        let Spect_GBetweenTimeStamps        = db.getSpect_GBetweenTimeStamps(fromTS, toTS);
        let Spect_HBetweenTimeStamps        = db.getSpect_HBetweenTimeStamps(fromTS, toTS);
        let Spect_IBetweenTimeStamps        = db.getSpect_IBetweenTimeStamps(fromTS, toTS);
        let Spect_JBetweenTimeStamps        = db.getSpect_JBetweenTimeStamps(fromTS, toTS);
        let Spect_KBetweenTimeStamps        = db.getSpect_KBetweenTimeStamps(fromTS, toTS);
        let Spect_LBetweenTimeStamps        = db.getSpect_LBetweenTimeStamps(fromTS, toTS);
        let Spect_RBetweenTimeStamps        = db.getSpect_RBetweenTimeStamps(fromTS, toTS);
        let Spect_SBetweenTimeStamps        = db.getSpect_SBetweenTimeStamps(fromTS, toTS);
        let Spect_TBetweenTimeStamps        = db.getSpect_TBetweenTimeStamps(fromTS, toTS);
        let Spect_UBetweenTimeStamps        = db.getSpect_UBetweenTimeStamps(fromTS, toTS);
        let Spect_VBetweenTimeStamps        = db.getSpect_VBetweenTimeStamps(fromTS, toTS);
        let Spect_WBetweenTimeStamps        = db.getSpect_WBetweenTimeStamps(fromTS, toTS);
        let buttonGreenBetweenTimeStamps    = db.getButton_GreenBetweenTimeStamps(fromTS, toTS);
        let buttonRedBetweenTimeStamps      = db.getButton_RedBetweenTimeStamps(fromTS, toTS);
        let phValueBetweenTimeStamps        = db.getPH_ValueBetweenTimeStamps(fromTS, toTS);
        let phVoltageBetweenTimeStamps      = db.getPH_VoltageBetweenTimeStamps(fromTS, toTS);
        let hr_confBetweenTimeStamps        = db.getHR_confidenceBetweenTimeStamps(fromTS, toTS);
        let hr_extStatBetweenTimeStamps     = db.getHR_extStatusBetweenTimeStamps(fromTS, toTS);
        let hr_statBetweenTimeStamps        = db.getHR_statusBetweenTimeStamps(fromTS, toTS);
        let hr_rateBetweenTimeStamps        = db.getHR_heartRateBetweenTimeStamps(fromTS, toTS);
        let hr_irLedBetweenTimeStamps       = db.getHR_irLedBetweenTimeStamps(fromTS, toTS);
        let hr_redLedBetweenTimeStamps      = db.getHR_redLedBetweenTimeStamps(fromTS, toTS);
        let hr_rValueBetweenTimeStamps      = db.getHR_rValueBetweenTimeStamps(fromTS, toTS);
        let avgWeightBetweenTimeStamps      = db.getScale_AvgWeightBetweenTimeStamps(fromTS, toTS);
        let weightBetweenTimeStamps         = db.getScale_WeightBetweenTimeStamps(fromTS, toTS);

        res.status(200).json({
            "decaWaveTagPositionsBetweenTimeStamps":decaWaveTagPositionsBetweenTimeStamps,
            "temperatureBetweenTimeStamps" : temperatureBetweenTimeStamps,
            "humidityBetweenTimeStamps" : humidityBetweenTimeStamps   ,
            "Spect_ABetweenTimeStamps" : Spect_ABetweenTimeStamps    ,
            "Spect_BBetweenTimeStamps" : Spect_BBetweenTimeStamps    ,
            "Spect_CBetweenTimeStamps" : Spect_CBetweenTimeStamps    ,
            "Spect_DBetweenTimeStamps" : Spect_DBetweenTimeStamps    ,
            "Spect_EBetweenTimeStamps" : Spect_EBetweenTimeStamps    ,
            "Spect_FBetweenTimeStamps" : Spect_FBetweenTimeStamps    ,
            "Spect_GBetweenTimeStamps" : Spect_GBetweenTimeStamps    ,
            "Spect_HBetweenTimeStamps" : Spect_HBetweenTimeStamps    ,
            "Spect_IBetweenTimeStamps" : Spect_IBetweenTimeStamps    ,
            "Spect_JBetweenTimeStamps" : Spect_JBetweenTimeStamps    ,
            "Spect_KBetweenTimeStamps" : Spect_KBetweenTimeStamps    ,
            "Spect_LBetweenTimeStamps" : Spect_LBetweenTimeStamps    ,
            "Spect_RBetweenTimeStamps" : Spect_RBetweenTimeStamps    ,
            "Spect_SBetweenTimeStamps" : Spect_SBetweenTimeStamps    ,
            "Spect_TBetweenTimeStamps" : Spect_TBetweenTimeStamps    ,
            "Spect_UBetweenTimeStamps" : Spect_UBetweenTimeStamps    ,
            "Spect_VBetweenTimeStamps" : Spect_VBetweenTimeStamps    ,
            "Spect_WBetweenTimeStamps" : Spect_WBetweenTimeStamps    ,
            "buttonGreenBetweenTimeStamps" : buttonGreenBetweenTimeStamps,
            "buttonRedBetweenTimeStamps" : buttonRedBetweenTimeStamps  ,
            "phValueBetweenTimeStamps" : phValueBetweenTimeStamps    ,
            "phVoltageBetweenTimeStamps" : phVoltageBetweenTimeStamps  ,
            "hr_confBetweenTimeStamps" : hr_confBetweenTimeStamps    ,
            "hr_extStatBetweenTimeStamps" : hr_extStatBetweenTimeStamps ,
            "hr_statBetweenTimeStamps" : hr_statBetweenTimeStamps    ,
            "hr_rateBetweenTimeStamps" : hr_rateBetweenTimeStamps    ,
            "hr_irLedBetweenTimeStamps" : hr_irLedBetweenTimeStamps   ,
            "hr_redLedBetweenTimeStamps" : hr_redLedBetweenTimeStamps  ,
            "hr_rValueBetweenTimeStamps" : hr_rValueBetweenTimeStamps  ,
            "avgWeightBetweenTimeStamps" : avgWeightBetweenTimeStamps  ,
            "weightBetweenTimeStamps" : weightBetweenTimeStamps     ,
        });
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

//region DecaWave
router.get('/decaWaveRawData', middleware.isLoggedIn, function (req, res) {
    let results = db.getDecaWaveRawData();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/decaWaveTagPositionsBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getDecaWaveTagPositionsBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/decaWaveTagPositionsAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getDecaWaveTagPositionsAfterTimeStamp(timeStamp, "b'mc'");
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/decaWaveRawData', middleware.isLoggedIn, function (req, res) {
    let timeStamp = req.body.timeStamp,
        msgType = req.body.msgType,
        srcType = req.body.srcType,
        anchor = req.body.anchor,
        tag = req.body.tag,
        range0 = req.body.range0,
        range1 = req.body.range1,
        range2 = req.body.range2,
        range3 = req.body.range3,
        rawData = req.body.rawData,
        x= req.body.position[0],
        y= req.body.position[1],
        z= req.body.position[2],
        position = "",
        position_kalman_calculated = req.body.position_kalman_calculated,
        position_kalman_filtered = req.body.position_kalman_filtered
    ;

    if (req.body.position_kalman_calculated != undefined  && req.body.position_kalman_calculated != undefined) {
        position =
            JSON.stringify({
                x:x, y:y, z:z,
                position_kalman_calculated_x: position_kalman_calculated[0],    position_kalman_calculated_y: position_kalman_calculated[1],
                position_kalman_filtered_x: position_kalman_filtered[0],        position_kalman_filtered_y: position_kalman_filtered[1],
            })
    } else {
        position = JSON.stringify({x:x, y:y, z:z,})
    }

    if (timeStamp !== "") {
        db.insertDecaWaveRawData(timeStamp, msgType, srcType, anchor, tag, range0, range1, range2, range3, position, rawData);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});

//endregion DecaWave

//region Atmospheric Sensor
//region Atmospheric Data
router.get('/atmosphericData', middleware.isLoggedIn, function (req, res) {
    let results = db.getAtmosphericData();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/atmosphericDataBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getAtmosphericDataBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/atmosphericDataAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getAtmosphericDataAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/atmosphericDataOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getAtmosphericDataOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/atmosphericDataOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getAtmosphericDataOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/atmosphericDataOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getAtmosphericDataOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/atmosphericData', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        temperature = parseFloat(req.body.temperature),
        humidity    = parseFloat(req.body.humidity),
        pressure    = parseFloat(req.body.pressure)
    ;

    if (timeStamp !== "") {
        db.insertAtmosphericData(timeStamp, deviceId, temperature, humidity, pressure);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Atmospheric Data

//region Temperature
router.get('/temperature', middleware.isLoggedIn, function (req, res) {
    let results = db.getTemperature();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperatureBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    console.log("temperatureBetweenTimeStamps: " + moment(fromTS).format("YYYY.MM.DD. HH:mm:ss") + " -> " + moment(toTS).format("YYYY.MM.DD. HH:mm:ss"));

    if (fromTS !== "" && toTS !== "") {
        let results = db.getTemperatureBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperatureAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getTemperatureAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperatureOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getTemperatureOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperatureOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getTemperatureOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperatureOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getTemperatureOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/temperature', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        temperature = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertTemperature(timeStamp, deviceId, temperature);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Temperature

//region Pressure
router.get('/pressure', middleware.isLoggedIn, function (req, res) {
    let results = db.getPressure();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/pressureBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getPressureBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/pressureAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getPressureAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/pressureOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getPressureOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/pressureOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getPressureOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/pressureOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getPressureOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/pressure', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        pressure    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertPressure(timeStamp, deviceId, pressure);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Pressure

//region Humidity
router.get('/humidity', middleware.isLoggedIn, function (req, res) {
    let results = db.getHumidity();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/humidityBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getHumidityBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/humidityAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getHumidityAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/humidityOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getHumidityOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/humidityOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getHumidityOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/humidityOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getHumidityOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/humidity', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        humidity    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertHumidity(timeStamp, deviceId, humidity);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Humidity
//endregion Atmospheric Sensor

//region Spectroscopy
//region Spect_A
router.get('/spect_A', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_A();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_ABetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_ABetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_AAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_AAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_AOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_AOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_AOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_AOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_AOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_AOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_A', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_A    = parseFloat(req.body.value).toFixed(3)
    ;

    console.log("Timestamp:" + timeStamp);

    if (timeStamp !== "") {
        db.insertSpect_A(timeStamp, deviceId, spect_A);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_A

//region Spect_B
router.get('/spect_B', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_B();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_BBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_BBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_BAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_BAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_BOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_BOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_BOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_BOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_BOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_BOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_B', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_B    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_B(timeStamp, deviceId, spect_B);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_B

//region Spect_C
router.get('/spect_C', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_C();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_CBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_CBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_CAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_CAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_COfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_COfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_COfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_COfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_COfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_COfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_C', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_C    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_C(timeStamp, deviceId, spect_C);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_C

//region Spect_D
router.get('/spect_D', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_D();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_DBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_DBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_DAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_DAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_DOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_DOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_DOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_DOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_DOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_DOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_D', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_D    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_D(timeStamp, deviceId, spect_D);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_D

//region Spect_E
router.get('/spect_E', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_E();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_EBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_EBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_EAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_EAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_EOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_EOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_EOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_EOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_EOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_EOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_E', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_E    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_E(timeStamp, deviceId, spect_E);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_E

//region Spect_F
router.get('/spect_F', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_F();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_FBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_FBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_FAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_FAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_FOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_FOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_FOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_FOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_FOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_FOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_F', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_F    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_F(timeStamp, deviceId, spect_F);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_F


//region Spect_G
router.get('/spect_G', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_G();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_GBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_GBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_GAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_GAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_GOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_GOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_GOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_GOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_GOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_GOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_G', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_G    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_G(timeStamp, deviceId, spect_G);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_G

//region Spect_H
router.get('/spect_H', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_H();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_HBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_HBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_HAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_HAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_HOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_HOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_HOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_HOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_HOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_HOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_H', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_H    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_H(timeStamp, deviceId, spect_H);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_H

//region Spect_I
router.get('/spect_I', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_I();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_IBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_IBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_IAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_IAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_IOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_IOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_IOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_IOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_IOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_IOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_I', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_I    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_I(timeStamp, deviceId, spect_I);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_I

//region Spect_J
router.get('/spect_J', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_J();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_JBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_JBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_JAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_JAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_JOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_JOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_JOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_JOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_JOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_JOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_J', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_J    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_J(timeStamp, deviceId, spect_J);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_J

//region Spect_K
router.get('/spect_K', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_K();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_KBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_KBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_KAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_KAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_KOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_KOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_KOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_KOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_KOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_KOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_K', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_K    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_K(timeStamp, deviceId, spect_K);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_K

//region Spect_L
router.get('/spect_L', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_L();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_LBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_LBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_LAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_LAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_LOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_LOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_LOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_LOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_LOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_LOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_L', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_L    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_L(timeStamp, deviceId, spect_L);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_L


//region Spect_R
router.get('/spect_R', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_R();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_RBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_RBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_RAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_RAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_ROfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_ROfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_ROfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_ROfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_ROfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_ROfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_R', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_R    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_R(timeStamp, deviceId, spect_R);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_R

//region Spect_S
router.get('/spect_S', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_S();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_SBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_SBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_SAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_SAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_SOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_SOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_SOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_SOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_SOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_SOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_S', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_S    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_S(timeStamp, deviceId, spect_S);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_S

//region Spect_T
router.get('/spect_T', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_T();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_TBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_TBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_TAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_TAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_TOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_TOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_TOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_TOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_TOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_TOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_T', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_T    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_T(timeStamp, deviceId, spect_T);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_T

//region Spect_U
router.get('/spect_U', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_U();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_UBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_UBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_UAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_UAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_UOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_UOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_UOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_UOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_UOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_UOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_U', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_U    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_U(timeStamp, deviceId, spect_U);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_U

//region Spect_V
router.get('/spect_V', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_V();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_VBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_VBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_VAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_VAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_VOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_VOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_VOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_VOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_VOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_VOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_V', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_V    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_V(timeStamp, deviceId, spect_V);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_V

//region Spect_W
router.get('/spect_W', middleware.isLoggedIn, function (req, res) {
    let results = db.getSpect_W();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_WBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getSpect_WBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_WAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getSpect_WAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_WOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getSpect_WOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/spect_WOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getSpect_WOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/spect_WOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getSpect_WOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/spect_W', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        spect_W    = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertSpect_W(timeStamp, deviceId, spect_W);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Spect_W
//endregion Spectroscopy

//region Buttons
//region Button_Green
router.get('/button_Green', middleware.isLoggedIn, function (req, res) {
    let results = db.getButton_Green();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/button_GreenBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getButton_GreenBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/button_GreenAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getButton_GreenAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/button_GreenOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getButton_GreenOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/button_GreenOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getButton_GreenOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/button_GreenOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getButton_GreenOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/button_Green', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        button_Green= parseInt(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertButton_Green(timeStamp, deviceId, button_Green);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Button_Green

//region Button_Red
router.get('/button_Red', middleware.isLoggedIn, function (req, res) {
    let results = db.getButton_Red();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/button_RedBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getButton_RedBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/button_RedAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getButton_RedAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/button_RedOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getButton_RedOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/button_RedOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getButton_RedOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/button_RedOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getButton_RedOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/button_Red', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        button_Red  = parseInt(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertButton_Red(timeStamp, deviceId, button_Red);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Button_Red
//endregion Buttons

//region PH Sensor
//region PH_Value
router.get('/ph_Value', middleware.isLoggedIn, function (req, res) {
    let results = db.getPH_Value();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/ph_ValueBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getPH_ValueBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/ph_ValueAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getPH_ValueAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/ph_ValueOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getPH_ValueOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/ph_ValueOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getPH_ValueOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/ph_ValueOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getPH_ValueOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/ph_Value', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        ph_Value= parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertPH_Value(timeStamp, deviceId, ph_Value);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion PH_Value

//region PH_Voltage
router.get('/ph_Voltage', middleware.isLoggedIn, function (req, res) {
    let results = db.getPH_Voltage();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/ph_VoltageBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getPH_VoltageBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/ph_VoltageAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getPH_VoltageAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/ph_VoltageOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getPH_VoltageOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/ph_VoltageOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getPH_VoltageOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/ph_VoltageOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getPH_VoltageOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/ph_Voltage', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        ph_Voltage  = parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertPH_Voltage(timeStamp, deviceId, ph_Voltage);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion PH_Voltage
//endregion PH Sensor

//region Heart Rate
//region HR_confidence
router.get('/hr_confidence', middleware.isLoggedIn, function (req, res) {
    let results = db.getHR_confidence();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_confidenceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getHR_confidenceBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_confidenceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getHR_confidenceAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_confidenceOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getHR_confidenceOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_confidenceOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getHR_confidenceOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_confidenceOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getHR_confidenceOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/hr_confidence', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        hr_confidence= parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertHR_confidence(timeStamp, deviceId, hr_confidence);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion HR_confidence

//region HR_extStatus
router.get('/hr_extStatus', middleware.isLoggedIn, function (req, res) {
    let results = db.getHR_extStatus();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_extStatusBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getHR_extStatusBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_extStatusAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getHR_extStatusAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_extStatusOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getHR_extStatusOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_extStatusOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getHR_extStatusOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_extStatusOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getHR_extStatusOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/hr_extStatus', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        hr_extStatus  = parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertHR_extStatus(timeStamp, deviceId, hr_extStatus);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion HR_extStatus

//region HR_heartRate
router.get('/hr_heartRate', middleware.isLoggedIn, function (req, res) {
    let results = db.getHR_heartRate();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_heartRateBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getHR_heartRateBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_heartRateAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getHR_heartRateAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_heartRateOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getHR_heartRateOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_heartRateOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getHR_heartRateOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_heartRateOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getHR_heartRateOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/hr_heartRate', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        hr_heartRate= parseFloat(req.body.value).toFixed(0)
    ;

    if (timeStamp !== "") {
        db.insertHR_heartRate(timeStamp, deviceId, hr_heartRate);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion HR_heartRate

//region HR_irLed
router.get('/hr_irLed', middleware.isLoggedIn, function (req, res) {
    let results = db.getHR_irLed();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_irLedBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getHR_irLedBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_irLedAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getHR_irLedAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_irLedOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getHR_irLedOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_irLedOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getHR_irLedOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_irLedOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getHR_irLedOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/hr_irLed', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        hr_irLed  = parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertHR_irLed(timeStamp, deviceId, hr_irLed);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion HR_irLed

//region HR_oxygen
router.get('/hr_oxygen', middleware.isLoggedIn, function (req, res) {
    let results = db.getHR_oxygen();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_oxygenBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getHR_oxygenBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_oxygenAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getHR_oxygenAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_oxygenOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getHR_oxygenOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_oxygenOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getHR_oxygenOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_oxygenOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getHR_oxygenOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/hr_oxygen', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        hr_oxygen= parseFloat(req.body.value).toFixed(0)
    ;

    if (timeStamp !== "") {
        db.insertHR_oxygen(timeStamp, deviceId, hr_oxygen);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion HR_oxygen

//region HR_redLed
router.get('/hr_redLed', middleware.isLoggedIn, function (req, res) {
    let results = db.getHR_redLed();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_redLedBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getHR_redLedBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_redLedAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getHR_redLedAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_redLedOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getHR_redLedOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_redLedOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getHR_redLedOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_redLedOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getHR_redLedOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/hr_redLed', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        hr_redLed  = parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertHR_redLed(timeStamp, deviceId, hr_redLed);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion HR_redLed

//region HR_rValue
router.get('/hr_rValue', middleware.isLoggedIn, function (req, res) {
    let results = db.getHR_rValue();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_rValueBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getHR_rValueBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_rValueAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getHR_rValueAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_rValueOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getHR_rValueOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_rValueOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getHR_rValueOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_rValueOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getHR_rValueOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/hr_rValue', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        hr_rValue= parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertHR_rValue(timeStamp, deviceId, hr_rValue);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion HR_rValue

//region HR_status
router.get('/hr_status', middleware.isLoggedIn, function (req, res) {
    let results = db.getHR_status();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_statusBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getHR_statusBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_statusAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getHR_statusAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_statusOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getHR_statusOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/hr_statusOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getHR_statusOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/hr_statusOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getHR_statusOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/hr_status', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        hr_status  = parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertHR_status(timeStamp, deviceId, hr_status);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion HR_status
//endregion Heart Rate

//region Scale
//region Scale_AvgWeight
router.get('/scale_AvgWeight', middleware.isLoggedIn, function (req, res) {
    let results = db.getScale_AvgWeight();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_AvgWeightBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getScale_AvgWeightBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_AvgWeightAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getScale_AvgWeightAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_AvgWeightOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getScale_AvgWeightOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_AvgWeightOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getScale_AvgWeightOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_AvgWeightOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getScale_AvgWeightOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/scale_AvgWeight', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        scale_AvgWeight= parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertScale_AvgWeight(timeStamp, deviceId, scale_AvgWeight);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Scale_AvgWeight

//region Scale_CalibrationDate
router.get('/scale_CalibrationDate', middleware.isLoggedIn, function (req, res) {
    let results = db.getScale_CalibrationDate();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_CalibrationDateBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getScale_CalibrationDateBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_CalibrationDateAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getScale_CalibrationDateAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_CalibrationDateOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getScale_CalibrationDateOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_CalibrationDateOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getScale_CalibrationDateOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_CalibrationDateOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getScale_CalibrationDateOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/scale_CalibrationDate', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        scale_CalibrationDate  = parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertScale_CalibrationDate(timeStamp, deviceId, scale_CalibrationDate);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Scale_CalibrationDate

//region Scale_CalibrationFactor
router.get('/scale_CalibrationFactor', middleware.isLoggedIn, function (req, res) {
    let results = db.getScale_CalibrationFactor();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_CalibrationFactorBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getScale_CalibrationFactorBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_CalibrationFactorAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getScale_CalibrationFactorAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_CalibrationFactorOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getScale_CalibrationFactorOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_CalibrationFactorOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getScale_CalibrationFactorOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_CalibrationFactorOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getScale_CalibrationFactorOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/scale_CalibrationFactor', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        scale_CalibrationFactor= parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertScale_CalibrationFactor(timeStamp, deviceId, scale_CalibrationFactor);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Scale_CalibrationFactor

//region Scale_Reading
router.get('/scale_Reading', middleware.isLoggedIn, function (req, res) {
    let results = db.getScale_Reading();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_ReadingBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getScale_ReadingBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_ReadingAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getScale_ReadingAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_ReadingOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getScale_ReadingOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_ReadingOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getScale_ReadingOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_ReadingOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getScale_ReadingOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/scale_Reading', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        scale_Reading  = parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertScale_Reading(timeStamp, deviceId, scale_Reading);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Scale_Reading

//region Scale_Weight
router.get('/scale_Weight', middleware.isLoggedIn, function (req, res) {
    let results = db.getScale_Weight();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_WeightBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getScale_WeightBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_WeightAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getScale_WeightAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_WeightOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getScale_WeightOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_WeightOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getScale_WeightOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_WeightOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getScale_WeightOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/scale_Weight', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        scale_Weight= parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.insertScale_Weight(timeStamp, deviceId, scale_Weight);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Scale_Weight

//region Scale_ZeroOffset
router.get('/scale_ZeroOffset', middleware.isLoggedIn, function (req, res) {
    let results = db.getScale_ZeroOffset();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_ZeroOffsetBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getScale_ZeroOffsetBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_ZeroOffsetAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getScale_ZeroOffsetAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_ZeroOffsetOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getScale_ZeroOffsetOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/scale_ZeroOffsetOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getScale_ZeroOffsetOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/scale_ZeroOffsetOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getScale_ZeroOffsetOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/scale_ZeroOffset', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        scale_ZeroOffset  = parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertScale_ZeroOffset(timeStamp, deviceId, scale_ZeroOffset);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion Scale_ZeroOffset
//endregion Scale

//region PWM
router.get('/pwm', middleware.isLoggedIn, function (req, res) {
    let results = db.getPWM();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/pwmBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getPWMBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/pwmAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getPWMAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/pwmOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getPWMOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/pwmOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getPWMOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/pwmOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getPWMOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/pwm', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        pwm= parseInt(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertPWM(timeStamp, deviceId, pwm);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion PWM

//region VoiceRecognition
router.get('/voiceRecognition', middleware.isLoggedIn, function (req, res) {
    let results = db.getVoiceRecognition();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/voiceRecognitionBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getVoiceRecognitionBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/voiceRecognitionAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getVoiceRecognitionAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/voiceRecognitionOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getVoiceRecognitionOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/voiceRecognitionOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getVoiceRecognitionOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/voiceRecognitionOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getVoiceRecognitionOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/voiceRecognition', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        voiceRecognition  = parseInt(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertVoiceRecognition(timeStamp, deviceId, voiceRecognition);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion VoiceRecognition

//region RFID
router.get('/rfid', middleware.isLoggedIn, function (req, res) {
    let results = db.getrfid();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/rfidBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getrfidBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/rfidAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getrfidAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/rfidOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getrfidOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/rfidOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getrfidOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/rfidOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getrfidOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(200).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/rfid', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        //value       = parseInt(req.body.value)
        value       = req.body.value
    ;

    if (timeStamp !== "") {db.insertrfid(timeStamp, deviceId, value);res.status(200).json({status: "Success"});}
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.get('/deleteRFIDOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;
    db.deleteRFIDOfDevice(deviceId);
    res.status(200).json({status: "OK", message: "Deleted"});
});

//endregion RFID

//region Battery
//region Voltage
router.get('/voltage', middleware.isLoggedIn, function (req, res) {
    let results = db.getvoltage();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/voltageBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getvoltageBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/voltageAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getvoltageAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/voltageOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getvoltageOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/voltageOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getvoltageOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/voltageOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getvoltageOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/voltage', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        voltage= parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertvoltage(timeStamp, deviceId, voltage);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion HR_heartRate

//region current
router.get('/current', middleware.isLoggedIn, function (req, res) {
    let results = db.getcurrent();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/currentBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.getcurrentBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/currentAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.getcurrentAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/currentOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.getcurrentOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/currentOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.getcurrentOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/currentOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.getcurrentOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/current', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        current= parseFloat(req.body.value)
    ;

    if (timeStamp !== "") {
        db.insertcurrent(timeStamp, deviceId, current);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion current
//endregion

//region Temperature
//region temperatureAmbient
router.get('/temperatureAmbient', middleware.isLoggedIn, function (req, res) {
    let results = db.gettemperatureAmbient();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperatureAmbientBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.gettemperatureAmbientBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperatureAmbientAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.gettemperatureAmbientAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperatureAmbientOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.gettemperatureAmbientOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperatureAmbientOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.gettemperatureAmbientOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperatureAmbientOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.gettemperatureAmbientOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/temperatureAmbient', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        temperatureAmbient= parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.inserttemperatureAmbient(timeStamp, deviceId, temperatureAmbient);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion temperatureAmbient

//region temperature_B_Ambient
router.get('/temperature_B_Ambient', middleware.isLoggedIn, function (req, res) {
    let results = db.gettemperature_B_Ambient();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperature_B_AmbientBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.gettemperature_B_AmbientBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_B_AmbientAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.gettemperature_B_AmbientAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_B_AmbientOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.gettemperature_B_AmbientOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperature_B_AmbientOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.gettemperature_B_AmbientOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_B_AmbientOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.gettemperature_B_AmbientOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/temperature_B_Ambient', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        temperature_B_Ambient= parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.inserttemperature_B_Ambient(timeStamp, deviceId, temperature_B_Ambient);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion temperature_B_Ambient

//region temperature_C_Ambient
router.get('/temperature_C_Ambient', middleware.isLoggedIn, function (req, res) {
    let results = db.gettemperature_C_Ambient();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperature_C_AmbientBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.gettemperature_C_AmbientBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_C_AmbientAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.gettemperature_C_AmbientAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_C_AmbientOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.gettemperature_C_AmbientOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperature_C_AmbientOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.gettemperature_C_AmbientOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_C_AmbientOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.gettemperature_C_AmbientOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/temperature_C_Ambient', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        temperature_C_Ambient= parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.inserttemperature_C_Ambient(timeStamp, deviceId, temperature_C_Ambient);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion temperature_C_Ambient

//region temperatureThermocouple
router.get('/temperatureThermocouple', middleware.isLoggedIn, function (req, res) {
    let results = db.gettemperatureThermocouple();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperatureThermocoupleBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.gettemperatureThermocoupleBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperatureThermocoupleAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.gettemperatureThermocoupleAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperatureThermocoupleOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.gettemperatureThermocoupleOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperatureThermocoupleOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.gettemperatureThermocoupleOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperatureThermocoupleOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.gettemperatureThermocoupleOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/temperatureThermocouple', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        temperatureThermocouple= parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.inserttemperatureThermocouple(timeStamp, deviceId, temperatureThermocouple);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion temperatureThermocouple

//region temperature_B_Thermocouple
router.get('/temperature_B_Thermocouple', middleware.isLoggedIn, function (req, res) {
    let results = db.gettemperature_B_Thermocouple();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperature_B_ThermocoupleBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.gettemperature_B_ThermocoupleBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_B_ThermocoupleAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.gettemperature_B_ThermocoupleAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_B_ThermocoupleOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.gettemperature_B_ThermocoupleOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperature_B_ThermocoupleOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.gettemperature_B_ThermocoupleOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_B_ThermocoupleOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.gettemperature_B_ThermocoupleOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/temperature_B_Thermocouple', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        temperature_B_Thermocouple= parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.inserttemperature_B_Thermocouple(timeStamp, deviceId, temperature_B_Thermocouple);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion temperature_B_Thermocouple

//region temperature_C_Thermocouple
router.get('/temperature_C_Thermocouple', middleware.isLoggedIn, function (req, res) {
    let results = db.gettemperature_C_Thermocouple();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperature_C_ThermocoupleBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);

    if (fromTS !== "" && toTS !== "") {
        let results = db.gettemperature_C_ThermocoupleBetweenTimeStamps(fromTS, toTS);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_C_ThermocoupleAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);

    if (timeStamp !== "") {
        let results = db.gettemperature_C_ThermocoupleAfterTimeStamp(timeStamp);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_C_ThermocoupleOfDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    let results = db.gettemperature_C_ThermocoupleOfDevice(deviceId);
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/temperature_C_ThermocoupleOfDeviceBetweenTimeStamps', middleware.isLoggedIn, function (req, res) {
    let fromTS = tryParseInt(req.query.from, 0);
    let toTS = tryParseInt(req.query.to, 0);
    let deviceId = req.query.deviceId;


    if (fromTS !== "" && toTS !== "" && deviceId !== "") {
        let results = db.gettemperature_C_ThermocoupleOfDeviceBetweenTimeStamps(fromTS, toTS, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.get('/temperature_C_ThermocoupleOfDeviceAfterTimeStamp', middleware.isLoggedIn, function (req, res) {
    let timeStamp = tryParseInt(req.query.timeStamp, 0);
    let deviceId = req.query.deviceId;

    if (timeStamp !== "" && deviceId !== "") {
        let results = db.gettemperature_C_ThermocoupleOfDeviceAfterTimeStamp(timeStamp, deviceId);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});
    } else res.status(400).json({status: "ERROR", message: "NotEnoughData"});
});

router.put('/temperature_C_Thermocouple', middleware.isLoggedIn, function (req, res) {

    let timeStamp   = req.body.timeStamp,
        deviceId    = req.body.deviceId,
        temperature_C_Thermocouple= parseFloat(req.body.value).toFixed(3)
    ;

    if (timeStamp !== "") {
        db.inserttemperature_C_Thermocouple(timeStamp, deviceId, temperature_C_Thermocouple);
        res.status(200).json({status: "Success"});
    } else {
        res.status(400).json({status: "ERROR", message: "NotEnoughData"});
    }
});
//endregion temperature_C_Thermocouple

//endregion

//region -= Enumerations =-

//region Status
router.get('/statuses', middleware.isLoggedIn, function (req, res) {
    let results = db.getStatuses();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.get('/statusesByRelevance', middleware.isLoggedIn, function (req, res) {
    let relevance = req.query.relevance;

    if(relevance !=="") {
        let results = db.getStatusesByRelevance(relevance);
        if (results.length > 0) res.status(200).json(results);
        else res.status(204).json({status: "ERROR", message: "DataNotFound"});
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}

});

router.put('/status', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id,
        name = req.query.name,
        description = req.query.description,
        relevance = req.query.relevance;
    if (id !== "" && name !== "") {
        let result = dataManager.insertOrModifyStatus(id, name, description, relevance);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});

router.delete('/status', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id;
    if (id !== "") {
        let result = dataManager.deleteStatus(id);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});
//endregion Status

//region Result
router.get('/results', middleware.isLoggedIn, function (req, res) {
    let results = db.getResults();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.put('/result', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id,
        name = req.query.name,
        description = req.query.description;
    if (id !== "" && name !== "") {
        let result = dataManager.insertOrModifyResult(id, name, description);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});

router.delete('/result', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id;
    if (id !== "") {
        let result = dataManager.deleteResult(id);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});
//endregion Result

//region Role
router.get('/roles', middleware.isLoggedIn, function (req, res) {
    let results = db.getRoles();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.put('/role', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id,
        name = req.query.name,
        description = req.query.description;
    if (id !== "" && name !== "") {
        let result = dataManager.insertOrModifyRole(id, name, description);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});

router.delete('/role', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id;
    if (id !== "") {
        let result = dataManager.deleteRole(id);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});
//endregion Role

//region userRole
router.get('/userRoles', middleware.isLoggedIn, function (req, res) {
    let results = db.getUserRoles();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.put('/userRole', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id,
        name = req.query.name,
        description = req.query.description;
    if (id !== "" && name !== "") {
        let result = dataManager.insertOrModifyUserRole(id, name, description);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});

router.delete('/userRole', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id;
    if (id !== "") {
        let result = dataManager.deleteUserRole(id);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});
//endregion Role

//region EventOperation
router.get('/eventOperations', middleware.isLoggedIn, function (req, res) {
    let results = db.getEventOperations();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.put('/eventOperation', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id,
        name = req.query.name,
        description = req.query.description;
    if (id !== "" && name !== "") {
        let result = dataManager.insertOrModifyEventOperation(id, name, description);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});

router.delete('/eventOperation', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id;
    if (id !== "") {
        let result = dataManager.deleteEventOperation(id);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});
//endregion EventOperation

//region EventResult
router.get('/eventResults', middleware.isLoggedIn, function (req, res) {
    let results = db.getEventResults();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.put('/eventResult', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id,
        name = req.query.name,
        description = req.query.description;
    if (id !== "" && name !== "") {
        let result = dataManager.insertOrModifyEventResult(id, name, description);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});

router.delete('/eventResult', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id;
    if (id !== "") {
        let result = dataManager.deleteEventResult(id);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});
//endregion EventResult

//region tag
router.get('/tags', middleware.isLoggedIn, function (req, res) {
    let results = db.getTags();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.put('/tag', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id,
        name = req.query.name,
        description = req.query.description;
    if (id !== "" && name !== "") {
        let result = dataManager.insertOrModifyTag(id, name, description);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});

router.delete('/tag', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id;
    if (id !== "") {
        let result = dataManager.deleteTag(id);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});
//endregion Role

//region DeviceType
router.get('/deviceTypes', middleware.isLoggedIn, function (req, res) {
    let results = db.getDeviceTypes();
    if (results.length > 0) res.status(200).json(results);
    else res.status(204).json({status: "ERROR", message: "DataNotFound"});
});

router.put('/deviceType', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id,
        name = req.query.name,
        description = req.query.description;
    if (id !== "" && name !== "") {
        let result = db.insertDeviceType(id, name, description);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});

router.delete('/deviceType', middleware.isLoggedIn, function (req, res) {
    let id = req.query.id;
    if (id !== "") {
        let result = dataManager.deleteDeviceType(id);
        eventLogger.insertEventWithResult(req.user.username, id, result);
        if(result.eventResult === eventLogger.eventResult.Success) res.status(200).json(result);
        else res.status(500).json(result);
    } else {res.status(400).json({status: "ERROR", message: "Kevés paraméter"});}
});
//endregion DeviceType

//endregion Enumerations

//region Device
router.get('/devices', middleware.isLoggedIn, function (req, res) {
    res.status(200).json(db.getDevices());
});

router.get('/devicesByDeviceType', middleware.isLoggedIn, function (req, res) {
    let deviceType = req.body.deviceType;
    res.status(200).json(db.getDevicesByDeviceType(deviceType));
});

router.get('/device', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;

    if (deviceId !== "") {
        let result = db.getDevice(deviceId);

        if (result != undefined && result.deviceId === deviceId) res.status(200).json(result);
        else res.status(204).json({status: "ERROR", message: "IdNotFound"});

    } else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.put('/device', middleware.isLoggedIn, function (req, res) {
    let deviceName = req.body.deviceName,
        MAC_address = req.body.MAC_address,
        deviceType = req.body.deviceType;

    if (deviceName !== "" && deviceType !== ""&& MAC_address !== "" ) {
        db.insertDevice(deviceName, deviceType, MAC_address);
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.put('/modifyDevice', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.body.deviceId,
        deviceName = req.body.deviceName,
        MAC_address = req.body.MAC_address,
        deviceType = req.body.deviceType;

    if (deviceId !== "" && deviceName !== ""&& deviceType !== ""&& MAC_address !== "") {
        db.modifyDevice(deviceId, deviceName, deviceType, MAC_address);
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.get('/updateDeviceId', middleware.isLoggedIn, function (req, res) {
    let deviceId_old = req.query.deviceId_old,
        deviceId_new = req.query.deviceId_new;

    if (deviceId_old !== "" && deviceId_new !== "") {
        db.updateDeviceId(deviceId_old, deviceId_new);
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.get('/updateDeviceIdBasedOnMacAddress', middleware.isLoggedIn, function (req, res) {
    let MACAddress = req.body.MACAddress,
        deviceId_new = req.body.deviceId_new;

    if (MACAddress !== "" && deviceId_new !== "") {
        db.updateDeviceIdBasedOnMacAddress(MACAddress, deviceId_new);
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});
router.get('/deleteWrongDevices', middleware.isLoggedIn, function (req, res) {
        db.deleteWrongDevices();
        res.status(200).json({status: "OK", message: "OK"});
});

router.delete('/device', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.body.deviceId;

    if (deviceId !== "") {
        db.deleteDevice(deviceId);
        res.status(200);
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});
//endregion

//region RFID_Device
router.get('/RFID_devices', middleware.isLoggedIn, function (req, res) {
    res.status(200).json(db.getRFIDDevices());
});

router.get('/RFID_device', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.body.deviceId;
    res.status(200).json(db.getRFIDDevice(deviceId));
});

router.get('/RFID_deviceByTime', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.query.deviceId;
    let timeStamp = tryParseInt(req.query.timeStamp);
    res.status(200).json(db.getRFIDDeviceByTimeStamp(deviceId, timeStamp));
});

router.put('/RFID_device', middleware.isLoggedIn, function (req, res) {
    let deviceId = req.body.deviceId,
        timeStampFrom = req.body.timeStampFrom,
        timeStampTo = req.body.timeStampTo,
        rfid = req.body.rfid;

    if (deviceId !== "" && rfid !== ""&& timeStampFrom !== ""&& timeStampTo !== "") {
        db.insertRFIDDevice(deviceId, rfid, timeStampFrom, timeStampTo );
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.put('/modifyRFID_Device', middleware.isLoggedIn, function (req, res) {
    let id = req.body.id,
        deviceId = req.body.deviceId,
        timeStampFrom = req.body.timeStampFrom,
        timeStampTo = req.body.timeStampTo,
        rfid = req.body.rfid;

    if (id !== "" && deviceId !== "" && rfid !== ""&& timeStampFrom !== ""&& timeStampTo !== "") {
        db.modifyRFIDDevice(id, deviceId, rfid, timeStampFrom, timeStampTo);
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});

router.delete('/RFID_device', middleware.isLoggedIn, function (req, res) {
    let id = req.body.id;

    if (id !== "") {
        db.deleteRFIDDevice(id);
        res.status(200).json({status: "OK", message: "OK"});
    }
    else {res.status(400).json({status: "ERROR", message: "NotEnoughData"});}
});
//endregion

module.exports = router;